#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "guiutils.h"

#include "cfg.h"
#include "convparm.h"
#include "winpreview.h"
#include "win.h"
#include "winanimicon.h"
#include "wincb.h"
#include "winlist.h"
#include "windnd.h"
#include "winopcb.h"
#include "winlistctrl.h"
#include "core.h"
#include "cfglist.h"
#include "config.h"

#include "images/icon_imgworks_20x20.xpm"
#include "images/icon_imgworks_48x48.xpm"

#include "images/icon_close_20x20.xpm"
#include "images/icon_new_20x20.xpm"
#include "images/icon_open_20x20.xpm"
#include "images/icon_cut_20x20.xpm"
#include "images/icon_copy_20x20.xpm"
#include "images/icon_paste_20x20.xpm"
#include "images/icon_cancel_20x20.xpm"
#include "images/icon_add_20x20.xpm"
#include "images/icon_remove_20x20.xpm"
#include "images/icon_clear_20x20.xpm"
#include "images/icon_properties_20x20.xpm"
#include "images/icon_rename_20x20.xpm"
#include "images/icon_folder_opened_20x20.xpm"
#include "images/icon_convert_20x20.xpm"
#include "images/icon_stop_20x20.xpm"
#include "images/icon_reload_20x20.xpm"
#include "images/icon_tool_bar_20x20.xpm"
#include "images/icon_status_bar_20x20.xpm"
#include "images/icon_image_browser_20x20.xpm"
#include "images/icon_help_20x20.xpm"
#include "images/icon_about_20x20.xpm"


/* Operation IDs */
win_opid_struct *WinOPIDNew(
	const win_opid id,
	win_struct *win
);
void WinOPIDDelete(win_opid_struct *opid);
GList *WinOPIDListNew(win_struct *win);
void WinOPIDListDelete(GList *glist);
win_opid_struct *WinOPIDListFind(GList *glist, win_opid id);

/* Target Directory */
void WinSetTarDir(win_struct *win, const gchar *path);
gchar *WinGetTarDir(win_struct *win);

/* Remove Converted */
void WinSetRemoveConverted(win_struct *win, gboolean remove_converted);
gboolean WinGetRemoveConverted(win_struct *win);

/* Recursive */
void WinSetRecursive(win_struct *win, gboolean recursive);
gboolean WinGetRecursive(win_struct *win);


/* Builders */
static void WinBuildMenuBar(win_struct *win);
static void WinBuildToolBar(win_struct *win);
static void WinBuildLists(win_struct *win);
static void WinBuildStatusBar(win_struct *win);
static void WinBuildListsMenu(win_struct *win);

/* Win */
win_struct *WinNew(gpointer core_ptr);
void WinSavePositions(win_struct *win);
void WinUpdate(win_struct *win);
void WinSetBusy(win_struct *win, gboolean busy);
void WinSetPassiveBusy(win_struct *win, gboolean busy);
gboolean WinToolBarIsShown(win_struct *win);
void WinToolBarSetShow(win_struct *win, gboolean show);
void WinToolBarUpdateItems(win_struct *win);
gboolean WinStatusBarIsShown(win_struct *win);
void WinStatusBarSetShow(win_struct *win, gboolean show);
void WinStatusProgress(
	win_struct *win, gfloat val, gboolean allow_gtk_iteration
);
void WinStatusMessage(
	win_struct *win, const gchar *mesg, gboolean allow_gtk_iteration
);
gboolean WinIsMapped(win_struct *win);
void WinMap(win_struct *win);
void WinUnmap(win_struct *win);
void WinDelete(win_struct *win);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)


#define WIN_WIDTH			800
#define WIN_HEIGHT			600
#define WIN_PANED1_POSITION		((gint)(WIN_WIDTH * 0.25f))
#define WIN_PANED2_POSITION		((gint)(WIN_WIDTH * 0.5f))
#define WIN_STATUS_BAR_HEIGHT		26


/*
 *	Creates a new Win OPID.
 */
win_opid_struct *WinOPIDNew(
	const win_opid id,
	win_struct *win
)
{
	win_opid_struct *opid = WIN_OPID(
	    g_malloc0(sizeof(win_opid_struct))
	);
	opid->id = id;
	opid->win = win;
	return(opid);
}

/*
 *	Deletes the Win OPID.
 */
void WinOPIDDelete(win_opid_struct *opid)
{
	if(opid == NULL)
	    return;

	g_free(opid->button_name);
	g_free(opid->menu_name);
	g_free(opid->tooltip);
	g_free(opid);
}

/*
 *	Creates a new Win OPID list.
 */
GList *WinOPIDListNew(win_struct *win)
{
	const gchar	*button_name,
			*menu_name,
			*tooltip;
	guint	accel_key,
		accel_mods;
	guint8	**button_icon_data,
		**menu_icon_data;
	ToolBarItemType item_type;
	win_opid_flags flags;
	void (*func_cb)(GtkWidget *, gpointer);
	GList *glist = NULL;

#define APPEND_OPID(_id_)	{			\
 win_opid_struct *opid = WinOPIDNew(		\
  (_id_), win					\
 );						\
 opid->flags = flags;				\
 opid->item_type = item_type;			\
 opid->button_name = STRDUP(button_name);	\
 opid->menu_name = STRDUP(menu_name);		\
 opid->tooltip = STRDUP(tooltip);		\
 opid->accel_key = accel_key;			\
 opid->accel_mods = accel_mods;			\
 opid->button_icon_data = button_icon_data;	\
 opid->menu_icon_data = menu_icon_data;		\
 opid->func_cb = func_cb;			\
 opid->enter_func_cb = WinOPIDEnterEventCB;	\
 opid->leave_func_cb = WinOPIDLeaveEventCB;	\
 glist = g_list_append(glist, opid);		\
}

	flags = WIN_OPID_ALLOW_MULTIPLE | WIN_OPID_NO_OP;
	item_type = TOOL_BAR_ITEM_SEPARATOR;
	button_name = NULL;
	menu_name = button_name;
	tooltip = NULL;
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = NULL;
	APPEND_OPID(WIN_OPID_SEPARATOR);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Close";
	menu_name = button_name;
	tooltip = "Close this window";
	accel_key = GDK_w;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_close_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinCloseCB;
	APPEND_OPID(WIN_OPID_CLOSE);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "New";
	menu_name = button_name;
	tooltip = "Remove all images from the source list";
	accel_key = GDK_n;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_new_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinNewCB;
	APPEND_OPID(WIN_OPID_NEW);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Add";
	menu_name = "Add..";
	tooltip = "Add object(s) to the source list";
	accel_key = GDK_Insert;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_add_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinAddCB;
	APPEND_OPID(WIN_OPID_ADD);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Remove";
	menu_name = "Remove";
	tooltip = "Remove the selected object from the source list";
	accel_key = GDK_Delete;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_remove_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinRemoveCB;
	APPEND_OPID(WIN_OPID_REMOVE);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Up";
	menu_name = "Shift Up";
	tooltip = "Shift the selected object(s) up";
	accel_key = '[';
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinShiftUpCB;
	APPEND_OPID(WIN_OPID_SHIFT_UP);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Down";
	menu_name = "Shift Down";
	tooltip = "Shift the selected object(s) down";
	accel_key = ']';
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinShiftDownCB;
	APPEND_OPID(WIN_OPID_SHIFT_DOWN);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Cut";
	menu_name = "Cut";
	tooltip = "Cut the selected object(s) to the clipboard";
	accel_key = GDK_x;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_cut_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinCutCB;
	APPEND_OPID(WIN_OPID_CUT);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Copy";
	menu_name = "Copy";
	tooltip = "Copy the selected object(s) to the clipboard";
	accel_key = GDK_c;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_copy_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinCopyCB;
	APPEND_OPID(WIN_OPID_COPY);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Paste";
	menu_name = "Paste";
	tooltip = "Paste any object from the clipboard";
	accel_key = GDK_v;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_paste_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinPasteCB;
	APPEND_OPID(WIN_OPID_PASTE);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Delete";
	menu_name = "Delete";
	tooltip = "Delete the selected object(s)";
	accel_key = GDK_Delete;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_cancel_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinRemoveCB;
	APPEND_OPID(WIN_OPID_DELETE);

	flags = 0; 
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Clear";
	menu_name = "Clear Source List";
	tooltip = "Remove all objects from the source list";
	accel_key = GDK_k;
	accel_mods = GDK_CONTROL_MASK | GDK_SHIFT_MASK;
	button_icon_data = (guint8 **)icon_clear_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinClearSrcCB;
	APPEND_OPID(WIN_OPID_CLEAR_SRC);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Clear";
	menu_name = "Clear Converted List";
	tooltip = "Remove all objects from the converted list";
	accel_key = GDK_k;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_clear_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinClearTarCB;
	APPEND_OPID(WIN_OPID_CLEAR_TAR);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "SelAll";
	menu_name = "Select All";
	tooltip = "Select all objects";
	accel_key = GDK_a;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinSelectAllCB;
	APPEND_OPID(WIN_OPID_SELECT_ALL);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "UnselAll";
	menu_name = "Unselect All";
	tooltip = "Unselect all objects";
	accel_key = GDK_u;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinUnselectAllCB;
	APPEND_OPID(WIN_OPID_UNSELECT_ALL);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "InvSel";
	menu_name = "Invert Selection";
	tooltip = "Select all unselected objects and unselected all\
 selected objects";
	accel_key = GDK_i;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinInvertSelectionCB;
	APPEND_OPID(WIN_OPID_INVERT_SELECTION);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Prop";
	menu_name = "Properties...";
	tooltip = "Modify the selected object's properties";
	accel_key = GDK_Return;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_properties_20x20_xpm;;
	menu_icon_data = button_icon_data;
	func_cb = WinPropertiesCB;
	APPEND_OPID(WIN_OPID_PROPERTIES);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON; 
	button_name = "TarDir";
	menu_name = "Set Target Directory...";
	tooltip = "Set the target directory";
	accel_key = GDK_t;
	accel_mods = GDK_CONTROL_MASK;
	button_icon_data = (guint8 **)icon_folder_opened_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinSetTarDirCB;
	APPEND_OPID(WIN_OPID_SET_TAR_DIR);


/* TODO */

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Convert";
	menu_name = "Convert";
	tooltip = "Convert the image(s)";
	accel_key = GDK_F8;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_convert_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinConvertCB;
	APPEND_OPID(WIN_OPID_CONVERT);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "ConvSel";
	menu_name = "Convert Selected";
	tooltip = "Convert the selected image(s)";
	accel_key = GDK_Return;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_convert_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinConvertSelectedCB;
	APPEND_OPID(WIN_OPID_CONVERT_SELECTED);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Stop";
	menu_name = "Stop";
	tooltip = "Stop the current operation";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_stop_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinStopCB;
	APPEND_OPID(WIN_OPID_STOP);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "RemConv";
	menu_name = "Remove Converted";
	tooltip = "Check this to remove the converted objects from the\
 source list after they have been converted";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinRemoveConvertedCB;
	APPEND_OPID(WIN_OPID_REMOVE_CONVERTED);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "Recurse";
	menu_name = "Recursive";
	tooltip = "Check this to recurse into subdirectories";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = NULL;
	menu_icon_data = button_icon_data;
	func_cb = WinRecursiveCB;
	APPEND_OPID(WIN_OPID_RECURSIVE);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Open";
	menu_name = button_name;
	tooltip = "Open the selected object";
	accel_key = GDK_Return;
	accel_mods = GDK_SHIFT_MASK;
	button_icon_data = (guint8 **)icon_open_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinOpenCB;
	APPEND_OPID(WIN_OPID_OPEN);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Refresh";
	menu_name = button_name;
	tooltip = "Refresh";
	accel_key = GDK_F5;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_reload_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinRefreshCB;
	APPEND_OPID(WIN_OPID_REFRESH);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "PViewSrc";
	menu_name = "Preview Source";
	tooltip = "Check this to preview source images";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_tool_bar_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinPreviewSrcCB;
	APPEND_OPID(WIN_OPID_PREVIEW_SRC);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "PVS Det";
	menu_name = "Show Details";
	tooltip = "Check this to show details with the preview of source images";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_tool_bar_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinPreviewSrcShowDetailsCB;
	APPEND_OPID(WIN_OPID_PREVIEW_SRC_SHOW_DETAILS);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "PViewConv";
	menu_name = "Preview Converted";
	tooltip = "Check this to preview converted images";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_tool_bar_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinPreviewTarCB;
	APPEND_OPID(WIN_OPID_PREVIEW_TAR);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "PVC Det";
	menu_name = "Show Details";
	tooltip = "Check this to show details with the preview of converted images";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_tool_bar_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinPreviewTarShowDetailsCB;
	APPEND_OPID(WIN_OPID_PREVIEW_TAR_SHOW_DETAILS);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "ToolBar";
	menu_name = "Tool Bar";
	tooltip = "Show/hide the tool bar";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_tool_bar_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinViewToolBarCB;
	APPEND_OPID(WIN_OPID_VIEW_TOOLBAR);

	flags = 0;
	item_type = TOOL_BAR_ITEM_TOGGLE_BUTTON;
	button_name = "StatusBar";
	menu_name = "Status Bar";
	tooltip = "Show/hide the status bar";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_status_bar_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinViewStatusBarCB;
	APPEND_OPID(WIN_OPID_VIEW_STATUSBAR);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "ImgWorks";
	menu_name = "New ImgWorks";
	tooltip = "Create a new ImgWorks";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_imgworks_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinNewImgWorksCB;
	APPEND_OPID(WIN_OPID_NEW_IMGWORKS);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "ImgBsr";
	menu_name = "New Image Browser";
	tooltip = "Create a new Image Browser";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_image_browser_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinNewImbrCB;
	APPEND_OPID(WIN_OPID_NEW_IMBR);


	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "Help";
	menu_name = "Contents";
	tooltip = "Index of all help pages";
	accel_key = GDK_F1;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_help_20x20_xpm;
	menu_icon_data = NULL;
	func_cb = WinHelpContentsCB;
	APPEND_OPID(WIN_OPID_HELP_CONTENTS);

	flags = 0;
	item_type = TOOL_BAR_ITEM_BUTTON;
	button_name = "About";
	menu_name = "About...";
	tooltip = "About this application";
	accel_key = 0;
	accel_mods = 0;
	button_icon_data = (guint8 **)icon_about_20x20_xpm;
	menu_icon_data = button_icon_data;
	func_cb = WinHelpAboutCB;
	APPEND_OPID(WIN_OPID_ABOUT);


#undef APPEND_OPID

	return(glist);
}

/*
 *	Deletes the Win OPID list.
 */
void WinOPIDListDelete(GList *glist)
{
	if(glist == NULL)
	    return;

	g_list_foreach(glist, (GFunc)WinOPIDDelete, NULL);
	g_list_free(glist);
}

/*
 *	Returns the Win OPID that matches the specified id in the
 *	list.
 */
win_opid_struct *WinOPIDListFind(GList *glist, win_opid id)
{
	win_opid_struct *opid;

	for(;
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	{
	    opid = WIN_OPID(glist->data);
	    if((opid != NULL) ? (opid->id == id) : FALSE)
		return(opid);
	}

	return(NULL);
}


/*
 *	Sets the target directory.
 */
void WinSetTarDir(win_struct *win, const gchar *path)
{
	GtkWidget *w;

	if(win == NULL)
	    return;

	win->freeze_count++;

	if(path != win->tar_dir)
	{
	    g_free(win->tar_dir);
	    win->tar_dir = STRDUP(path);
	}

	w = win->tar_dir_entry;
	if(w != NULL)
	    gtk_entry_set_text(
		GTK_ENTRY(w),
		(path != NULL) ? path : ""
	    );

	win->freeze_count--;
}

/*
 *	Gets the target directory.
 */
gchar *WinGetTarDir(win_struct *win)
{
	return((win != NULL) ? win->tar_dir : NULL);
}


/*
 *	Sets the remove converted setting.
 */
void WinSetRemoveConverted(win_struct *win, gboolean remove_converted)
{
	if(win == NULL)
	    return;

	if(remove_converted == win->remove_converted)
	    return;

	win->freeze_count++;

	win->remove_converted = remove_converted;

	GUIMenuItemSetCheck(
	    win->remove_converted_micheck, remove_converted, TRUE
	);

	win->freeze_count--;
}

/*
 *	Gets the remove converted setting.
 */
gboolean WinGetRemoveConverted(win_struct *win)
{
	return((win != NULL) ? win->remove_converted : FALSE);
}


/*
 *	Sets the recursive setting.
 */
void WinSetRecursive(win_struct *win, gboolean recursive)
{
	if(win == NULL)
	    return;

	if(recursive == win->recursive)
	    return;

	win->freeze_count++;

	win->recursive = recursive;

	GUIMenuItemSetCheck(
	    win->recursive_micheck, recursive, TRUE
	);

	win->freeze_count--;
}

/*
 *	Gets the recursive setting.
 */
gboolean WinGetRecursive(win_struct *win)
{
	return((win != NULL) ? win->recursive : FALSE);
}


/*
 *	Builds the Win's Menu Bar.
 */
static void WinBuildMenuBar(win_struct *win)
{
	GtkAccelGroup *accelgrp = win->accelgrp;
	GtkWidget *w, *parent, *menubar, *menu;

	parent = win->menubar_handle;


	/* Recreate the Menu Bar */
	GTK_WIDGET_DESTROY(win->menubar)
	win->menubar = menubar = w = GUIMenuBarCreate(NULL);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);

#define ADD_MENU_ITEM_LABEL(_id_)	{	\
 win_opid_struct *opid = WinOPIDListFind(	\
  win->opid, (_id_)				\
 );						\
 if(opid != NULL) {				\
  w = GUIMenuItemCreate(			\
   menu,					\
   GUI_MENU_ITEM_TYPE_LABEL,			\
   accelgrp,					\
   opid->menu_icon_data,			\
   opid->menu_name,				\
   opid->accel_key, opid->accel_mods,		\
   opid->func_cb, opid->win			\
  );						\
  GUISetMenuItemCrossingCB(			\
   w,						\
   (gpointer)opid->enter_func_cb, opid,		\
   (gpointer)opid->leave_func_cb, opid		\
  );						\
 }						\
 else						\
  w = NULL;					\
}

#define ADD_MENU_ITEM_CHECK(_id_)	{	\
 win_opid_struct *opid = WinOPIDListFind(	\
  win->opid, (_id_)				\
 );						\
 if(opid != NULL) {				\
  w = GUIMenuItemCreate(			\
   menu,					\
   GUI_MENU_ITEM_TYPE_CHECK,			\
   accelgrp,					\
   opid->menu_icon_data,			\
   opid->menu_name,				\
   opid->accel_key, opid->accel_mods,		\
   opid->func_cb, opid->win			\
  );						\
  GUISetMenuItemCrossingCB(			\
   w,						\
   (gpointer)opid->enter_func_cb, opid,		\
   (gpointer)opid->leave_func_cb, opid		\
  );						\
 }						\
 else						\
  w = NULL;					\
}

#define ADD_MENU_ITEM_SEP	{		\
 w = GUIMenuItemCreate(				\
  menu,						\
  GUI_MENU_ITEM_TYPE_SEPARATOR,			\
  NULL,						\
  NULL,						\
  NULL,						\
  0, 0,						\
  NULL, NULL					\
 );						\
}

	/* Create the File menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
	    ADD_MENU_ITEM_LABEL(WIN_OPID_NEW);
	    win->new_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_ADD);
	    win->file_add_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_REMOVE);
	    win->file_remove_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_CLOSE);
	    win->close_mi = w;
	}
	GUIMenuAddToMenuBar(
	    menubar, menu,
#if defined(PROG_LANGUAGE_SPANISH)
"Archivo"
#elif defined(PROG_LANGUAGE_FRENCH)
"Fichier"
#elif defined(PROG_LANGUAGE_GERMAN)
"Akte"
#elif defined(PROG_LANGUAGE_ITALIAN)
"File"
#elif defined(PROG_LANGUAGE_DUTCH)
"Dossier"
#elif defined(PROG_LANGUAGE_PORTUGUESE)
"Arquivo"
#elif defined(PROG_LANGUAGE_NORWEGIAN)
"Arkiv"
#else
"File"
#endif
	    , GTK_JUSTIFY_LEFT
	);

	/* Create the Edit menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
	    ADD_MENU_ITEM_LABEL(WIN_OPID_ADD);
	    win->add_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_REMOVE);
	    win->remove_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_SHIFT_UP);
	    win->shift_up_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_SHIFT_DOWN);
	    win->shift_down_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_CUT);
	    win->cut_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_COPY);
	    win->copy_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_PASTE);
	    win->paste_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_DELETE);
	    win->delete_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_CLEAR_SRC);
	    win->clear_src_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_CLEAR_TAR);
	    win->clear_tar_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_SELECT_ALL);
	    win->select_all_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_UNSELECT_ALL);
	    win->unselect_all_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_INVERT_SELECTION);
	    win->invert_selection_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_PROPERTIES);
	    win->properties_mi = w;
	}
	GUIMenuAddToMenuBar(
	    menubar, menu,
#if defined(PROG_LANGUAGE_SPANISH)
"Redacte"
#elif defined(PROG_LANGUAGE_FRENCH)
"Editer"
#elif defined(PROG_LANGUAGE_GERMAN)
"Redigieren"
#elif defined(PROG_LANGUAGE_ITALIAN)
"Redigere"
#elif defined(PROG_LANGUAGE_DUTCH)
"Bewerking"
#elif defined(PROG_LANGUAGE_PORTUGUESE)
"Edite"
#elif defined(PROG_LANGUAGE_NORWEGIAN)
"Rediger"
#else
"Edit"
#endif
	    , GTK_JUSTIFY_LEFT
	);

	/* Create the View menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
	    ADD_MENU_ITEM_LABEL(WIN_OPID_OPEN);
	    win->open_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_REFRESH);
	    win->refresh_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_CHECK(WIN_OPID_PREVIEW_SRC);
	    win->preview_src_micheck = w;
	    ADD_MENU_ITEM_CHECK(WIN_OPID_PREVIEW_SRC_SHOW_DETAILS);
	    win->preview_src_show_details_micheck = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_CHECK(WIN_OPID_PREVIEW_TAR);
	    win->preview_tar_micheck = w;
	    ADD_MENU_ITEM_CHECK(WIN_OPID_PREVIEW_TAR_SHOW_DETAILS);
	    win->preview_tar_show_details_micheck = w;

	    ADD_MENU_ITEM_SEP 

	    ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_TOOLBAR);
	    win->view_tool_bar_micheck = w;
	    ADD_MENU_ITEM_CHECK(WIN_OPID_VIEW_STATUSBAR);
	    win->view_statusbar_micheck = w;
	}
	GUIMenuAddToMenuBar( 
	    menubar, menu,
#if defined(PROG_LANGUAGE_SPANISH)
"Vista"
#elif defined(PROG_LANGUAGE_FRENCH)
"Vue"
#elif defined(PROG_LANGUAGE_GERMAN)
"Blick"
#elif defined(PROG_LANGUAGE_ITALIAN)
"Veduta"
#elif defined(PROG_LANGUAGE_DUTCH)
"Overzicht"
#elif defined(PROG_LANGUAGE_PORTUGUESE)
"Vista"
#elif defined(PROG_LANGUAGE_NORWEGIAN)
"Sikt"
#else
"View"
#endif
	    , GTK_JUSTIFY_LEFT
	);

	/* Create the Target menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
	    ADD_MENU_ITEM_LABEL(WIN_OPID_SET_TAR_DIR);
	    win->set_tar_dir_mi = w;
	}
	GUIMenuAddToMenuBar(
	    menubar, menu, "Target", GTK_JUSTIFY_LEFT
	);

	/* Create the Convert menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
	    ADD_MENU_ITEM_LABEL(WIN_OPID_CONVERT);
	    win->convert_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_CONVERT_SELECTED);
	    win->convert_selected_mi = w;
	    ADD_MENU_ITEM_LABEL(WIN_OPID_STOP);
	    win->stop_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_CHECK(WIN_OPID_REMOVE_CONVERTED);
	    win->remove_converted_micheck = w;
	    ADD_MENU_ITEM_CHECK(WIN_OPID_RECURSIVE);
	    win->recursive_micheck = w;
	}
	GUIMenuAddToMenuBar(
	    menubar, menu, "Convert", GTK_JUSTIFY_LEFT
	);

	/* Create the Windows menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
            ADD_MENU_ITEM_LABEL(WIN_OPID_NEW_IMGWORKS);
            win->new_imgworks_mi = w;

	    ADD_MENU_ITEM_LABEL(WIN_OPID_NEW_IMBR);
	    win->new_imbr_mi = w;
	}
	GUIMenuAddToMenuBar( 
	    menubar, menu, 
	    "Windows", GTK_JUSTIFY_LEFT
	);

	/* Create the Help menu */
	menu = GUIMenuCreateTearOff();
	if(menu != NULL)
	{
	    ADD_MENU_ITEM_LABEL(WIN_OPID_HELP_CONTENTS);
	    win->help_contents_mi = w;

	    ADD_MENU_ITEM_SEP

	    ADD_MENU_ITEM_LABEL(WIN_OPID_ABOUT);
	    win->about_mi = w;
	}
	GUIMenuAddToMenuBar( 
	    menubar, menu,
#if defined(PROG_LANGUAGE_SPANISH)
"Ayuda"
#elif defined(PROG_LANGUAGE_FRENCH)
"Aide"
#elif defined(PROG_LANGUAGE_GERMAN)
"Hilfe"
#elif defined(PROG_LANGUAGE_ITALIAN)
"L'Aiuto"
#elif defined(PROG_LANGUAGE_DUTCH)
"Hulp"
#elif defined(PROG_LANGUAGE_PORTUGUESE)
"Ajuda"
#elif defined(PROG_LANGUAGE_NORWEGIAN)
"Hjelp"
#else
"Help"
#endif
	    , GTK_JUSTIFY_RIGHT
	);

#undef ADD_MENU_ITEM_LABEL
#undef ADD_MENU_ITEM_CHECK
#undef ADD_MENU_ITEM_SEP
}

/*
 *	Builds the Win's Tool Bar.
 */
static void WinBuildToolBar(win_struct *win)
{
	const gint border_major = 5;
	GList		*glist,
			*tb_cfg,
			*items_list;
	GtkWidget	*parent,
			*tb;
	ToolBarButtonDecals tb_display = TOOL_BAR_BUTTON_DECALS_PICTURES;
	GtkReliefStyle tb_relief = GTK_RELIEF_NONE;
	ToolBarItem *item;
	win_opid_struct *opid;
/*	core_struct *core = win->core; */

	parent = win->tool_bar_handle;


	/* Create the Tool Bar configuration list */
	glist = NULL;
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_ADD
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_REMOVE
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_CLEAR_SRC
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SEPARATOR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_CUT
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_COPY
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_PASTE
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SEPARATOR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_PROPERTIES
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SEPARATOR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SET_TAR_DIR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SEPARATOR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_CONVERT
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_STOP
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SEPARATOR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_CLEAR_TAR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_SEPARATOR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_OPEN
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_NEW_IMBR
	);
	glist = g_list_append(
	    glist, (gpointer)WIN_OPID_NEW_IMGWORKS
	);
	tb_cfg = glist;

	/* Create Tool Bar Items list from the Tool Bar configuration */
	items_list = NULL;
	for(glist = tb_cfg;
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	{
	    /* Get Operation ID from the Tool Bar configuration which
	     * specifies the ID
	     */
	    opid = WinOPIDListFind(win->opid, (win_opid)glist->data);
	    if(opid == NULL)
		continue;

	    item = ToolBarItemNew(
		opid->item_type,
		opid->button_name,
		opid->button_icon_data,
		opid->tooltip,
		opid->id,
		WinToolBarItemCB, opid,
		WinToolBarItemEnterCB, opid,
		WinToolBarItemLeaveCB, opid
	    );
	    items_list = g_list_append(
		items_list,
		item
	    );
	}

	/* Recreate the Tool Bar */
	GTK_WIDGET_DESTROY(win->tool_bar);
	win->tool_bar = tb = ToolBarNew(
	    GTK_ORIENTATION_HORIZONTAL,
	    tb_display,
	    tb_relief,
	    GTK_POLICY_AUTOMATIC,
	    items_list
	);
	gtk_container_add(GTK_CONTAINER(parent), tb);
	gtk_container_border_width(GTK_CONTAINER(tb), border_major);
	gtk_widget_show(tb);

	/* Delete the Tool Bar items list */
	if(items_list != NULL)
	{
	    g_list_foreach(
		items_list,
		(GFunc)ToolBarItemDelete,
		NULL
	    );
	    g_list_free(items_list);
	}

	/* Delete the Tool Bar configuration list */
	g_list_free(tb_cfg);
}

/*
 *	Builds the Win's Lists.
 */
static void WinBuildLists(win_struct *win)
{
	const gint	border_major = 5,
			border_minor = 2;
	const gint columns = 1;
	const GtkTargetEntry dnd_tar_types[] = {
{GUI_TARGET_NAME_TEXT_PLAIN,	0,		DND_INFO_TEXT_PLAIN},
{GUI_TARGET_NAME_TEXT_URI_LIST,	0,		DND_INFO_TEXT_URI_LIST},
{GUI_TARGET_NAME_STRING,	0,		DND_INFO_STRING}
	};
	const GtkTargetEntry dnd_src_types[] = {
{GUI_TARGET_NAME_TEXT_PLAIN,	0,		DND_INFO_TEXT_PLAIN},
{GUI_TARGET_NAME_TEXT_URI_LIST,	0,		DND_INFO_TEXT_URI_LIST},
{GUI_TARGET_NAME_STRING,	0,		DND_INFO_STRING}
	};
	gint i;
	gchar **strv;
	GtkWidget *w, *parent, *parent2, *parent3, *parent4, *parent5;
	GtkCList *clist;
	convparm_struct *cp;
	core_struct *core = CORE(win->core);
	CfgItem *cfg_list = core->cfg_list;

	parent = win->lists_paned1;


	/* Left side of paned */

	w = gtk_vbox_new(FALSE, border_major);
	gtk_paned_add1(GTK_PANED(parent), w);
	gtk_widget_show(w);
	parent2 = w;

	/* GtkScrolledWindow for the GtkCList */
	w = gtk_scrolled_window_new(NULL, NULL); 
	gtk_scrolled_window_set_policy(
	    GTK_SCROLLED_WINDOW(w),
	    GTK_POLICY_AUTOMATIC,
	    GTK_POLICY_AUTOMATIC
	);
	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent3 = w;

	/* Source GtkCList */
	strv = (gchar **)g_malloc(columns * sizeof(gchar *));
	for(i = 0; i < columns; i++)
	    strv[i] = "";
	win->src_clist = w = gtk_clist_new_with_titles(
	    columns, strv
	);
	g_free(strv);
	clist = GTK_CLIST(w);
	gtk_widget_add_events(
	    w,
	    GDK_FOCUS_CHANGE_MASK |
	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
	    GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
	    GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_press_event",
	    GTK_SIGNAL_FUNC(WinCListKeyEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_release_event",
	    GTK_SIGNAL_FUNC(WinCListKeyEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_press_event",
	    GTK_SIGNAL_FUNC(WinCListButtonEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_release_event",
	    GTK_SIGNAL_FUNC(WinCListButtonEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "motion_notify_event",
	    GTK_SIGNAL_FUNC(WinCListMotionEventCB), core
	);
	gtk_signal_connect(  
	    GTK_OBJECT(w), "focus_in_event",
	    GTK_SIGNAL_FUNC(WinSrcListEventCB), win  
	);
	gtk_signal_connect(  
	    GTK_OBJECT(w), "focus_out_event",
	    GTK_SIGNAL_FUNC(WinSrcListEventCB), win  
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_press_event",
	    GTK_SIGNAL_FUNC(WinSrcListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_release_event",
	    GTK_SIGNAL_FUNC(WinSrcListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_press_event",
	    GTK_SIGNAL_FUNC(WinSrcListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_release_event",
	    GTK_SIGNAL_FUNC(WinSrcListEventCB), win
	);
	gtk_container_add(GTK_CONTAINER(parent3), w);
	gtk_widget_realize(w);
	gtk_clist_set_selection_mode(clist, GTK_SELECTION_EXTENDED);
	for(i = 0; i < columns; i++)
	    gtk_clist_set_column_width(clist, i, 10);
	gtk_clist_column_titles_passive(clist);
	gtk_clist_set_column_title(clist, 0, "Source Images");
	gtk_clist_set_row_height(clist, WIN_LIST_ROW_SPACING);
	gtk_clist_set_shadow_type(clist, GTK_SHADOW_IN);
	gtk_signal_connect(
	    GTK_OBJECT(w), "resize_column",
	    GTK_SIGNAL_FUNC(WinSrcListResizeColumnCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "click_column",
	    GTK_SIGNAL_FUNC(WinSrcListClickColumnCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "select_row",
	    GTK_SIGNAL_FUNC(WinSrcListSelectRowCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "unselect_row",
	    GTK_SIGNAL_FUNC(WinSrcListUnselectRowCB), win
	);
	GUIDNDSetSrc(
	    w,
	    dnd_src_types,
	    sizeof(dnd_src_types) / sizeof(GtkTargetEntry),
	    GDK_ACTION_MOVE | GDK_ACTION_COPY |	/* Actions */
		GDK_ACTION_LINK,
	    GDK_BUTTON1_MASK,			/* Buttons */
	    NULL,
	    WinCListDragDataGetCB,
	    WinCListDragDataDeleteCB,
	    NULL,
	    win
	); 
	GUIDNDSetTar(
	    w,
	    dnd_tar_types,
	    sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
	    GDK_ACTION_MOVE,			/* Actions */ 
	    GDK_ACTION_MOVE,			/* Default action if same */
	    GDK_ACTION_MOVE,			/* Default action */
	    WinCListDragDataReceivedCB,
	    win,
	    TRUE				/* Highlight */
	);
	gtk_widget_show(w);


	/* Preview */
	win->src_preview = w = WinPreviewNew();
	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_widget_realize(w);
	WinPreviewSetSize(w, -1, 100);
	WinPreviewSetShowDetails(w, win->preview_src_show_details);
	GUIDNDSetTar(
	    WinPreviewGetDrawingArea(w),
	    dnd_tar_types,
	    sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
	    GDK_ACTION_MOVE,			/* Actions */
	    GDK_ACTION_MOVE,			/* Default action if same */
	    GDK_ACTION_MOVE,			/* Default action */
	    WinPreviewDragDataReceivedCB,
	    win,
	    TRUE				/* Highlight */
	);
	if(win->preview_src)
	    gtk_widget_show(w);


	/* Center of paned */

	win->lists_paned2 = w = gtk_hpaned_new();
	if(cfg_list != NULL)
	{
	    gint x = CFG_GET_I(CFG_PARM_WIN_PANED2_POSITION);
	    if(x <= 0)
		x = WIN_PANED2_POSITION;
	    gtk_paned_set_position(GTK_PANED(w), x);
	}
	gtk_paned_set_handle_size(GTK_PANED(w), 10);
	gtk_paned_set_gutter_size(GTK_PANED(w), 10);
	gtk_paned_add2(GTK_PANED(parent), w);
	gtk_widget_show(w);
	parent = w;

	w = gtk_vbox_new(FALSE, border_major);
	gtk_paned_add1(GTK_PANED(parent), w);
	gtk_widget_show(w);
	parent2 = w;

	/* GtkVBox for the conversion parameters heading and 
	 * GtkScrolledWindow
	 */
	w = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent3 = w;

	/* Conversion Parameters list heading label */
	w = gtk_hbox_new(FALSE, border_major);
	gtk_container_border_width(GTK_CONTAINER(w), border_major);
	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent4 = w;
	/* GtkLabel */
	w = gtk_label_new("Conversion Parameters");
	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
	gtk_widget_show(w);

	/* GtkScrolledWindow for the Conversion Parameters list */
	w = WinScrolledWindowNew(
	    core,
	    GTK_POLICY_AUTOMATIC,
	    GTK_POLICY_AUTOMATIC,
	    NULL, &parent4
	);
	gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
	gtk_widget_show(w);

	w = parent4;
	gtk_container_border_width(GTK_CONTAINER(w), border_major);
	gtk_box_set_spacing(GTK_BOX(w), border_major);

	/* Begin creating Conversion Parameters
	 *
	 * Note that the order that the conversion parameters are 
	 * created here determine the order at which they are
	 * followed by in the conersion process
	 */

	/* Format */
	cp = ConvParmFmtNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	ConvParmFmtSet(cp, "*");
	ConvParmEnable(cp);
	ConvParmShow(cp);

	/* Compression */
	cp = ConvParmCompressionNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	ConvParmCompressionSet(cp, 1.0f);
	ConvParmEnable(cp);
	ConvParmShow(cp);

	/* Background Color */ 
	cp = ConvParmColorBGNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp); 
	if(cp != NULL)
	{
	    GdkColor c = { 0l, 0x0000, 0x0000, 0x0000 };
	    ConvParmColorBGSet(cp, &c);
	}
	ConvParmDisable(cp);
	ConvParmShow(cp);

	/* Gamma */
	cp = ConvParmGammaNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	ConvParmGammaSet(cp, 1.0f);
	ConvParmDisable(cp);
	ConvParmShow(cp);

	/* Mirror */
	cp = ConvParmMirrorNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	ConvParmMirrorSet(cp, GTK_ORIENTATION_HORIZONTAL);
	ConvParmDisable(cp);
	ConvParmShow(cp);

	/* Crop */
	cp = ConvParmCropNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	if(cp != NULL)
	{
	    GdkRectangle rect;
	    rect.x = 0;
	    rect.y = 0;
	    rect.width = 320;
	    rect.height = 240;
	    ConvParmCropSet(cp, &rect);
	}
	ConvParmDisable(cp);
	ConvParmShow(cp);

	/* Rotate */
	cp = ConvParmRotateNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	ConvParmRotateSet(cp, 0.0f);
	ConvParmDisable(cp);
	ConvParmShow(cp);

	/* Size */
	cp = ConvParmSizeNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	if(cp != NULL)
	{
	    convparm_sizeinfo_struct s;
	    s.width = 320;
	    s.height = 240;
	    s.maintain_aspect = TRUE;
	    s.smaller = TRUE;
	    ConvParmSizeSet(cp, &s);
	}
	ConvParmDisable(cp);
	ConvParmShow(cp);

	/* Frame */
	cp = ConvParmFrameNew(parent4);
	win->convparm_list = g_list_append(win->convparm_list, cp);
	if(cp != NULL)
	{
	    convparm_frameinfo_struct f;
	    GdkColor c = { 0, 0x0000, 0x0000, 0x0000 };
	    f.width = 3;
	    f.height = 3;
	    f.inner_bevel = 1;
	    f.outer_bevel = 1;
	    memcpy(&f.c, &c, sizeof(GdkColor));
	    ConvParmFrameSet(cp, &f);
	}
	ConvParmDisable(cp);
	ConvParmShow(cp);


/* TODO */



	w = gtk_hbox_new(FALSE, border_major);
	gtk_container_border_width(GTK_CONTAINER(w), border_major);
	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent3 = w;

	w = gtk_vbox_new(FALSE, border_major);
	gtk_box_pack_start(GTK_BOX(parent3), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent4 = w;

	/* Target Directory */
	w = gtk_hbox_new(FALSE, border_minor);
	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent5 = w;
	/* GtkLabel */
	w = gtk_label_new("Target:");
	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	/* GtkEntry */
	win->tar_dir_entry = w = gtk_entry_new();
	gtk_box_pack_start(GTK_BOX(parent5), w, TRUE, TRUE, 0);
        gtk_signal_connect(
            GTK_OBJECT(w), "key_press_event",
            GTK_SIGNAL_FUNC(WinEntryEntryCompletePathKeyCB), win
        );
        gtk_signal_connect(
            GTK_OBJECT(w), "key_release_event",
            GTK_SIGNAL_FUNC(WinEntryEntryCompletePathKeyCB), win
        );
	gtk_signal_connect(
	    GTK_OBJECT(w), "changed",
	    GTK_SIGNAL_FUNC(WinAnyChangedCB), win
	);
	GUIEditableEndowPopupMenu(w, 0);
	GUISetWidgetTip(
	    w,
"Enter the directory that you want to place all of the converted\
 images in, or leave this blank to place each converted image in the\
 same directory as its source image"
	);
	gtk_widget_show(w);
	/* Browse Button */
	win->tar_dir_browse_btn = w = GUIButtonPixmap(
	    (guint8 **)icon_folder_opened_20x20_xpm
	);
	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(WinSetTarDirCB), win
	);
	GUISetWidgetTip(w, "Browse");
	gtk_widget_show(w);
	/* Clear Button */
	win->tar_dir_clear_btn = w = GUIButtonPixmap(
	    (guint8 **)icon_clear_20x20_xpm
	);
	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(WinEntryClearCB), win->tar_dir_entry
	);
	GUISetWidgetTip(w, "Clear");
	gtk_widget_show(w);


	/* GtkHBox for the convert buttons */
	w = gtk_hbox_new(TRUE, 0);
	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent5 = w;

	/* Convert Button */
	win->convert_btn = w = GUIButtonPixmapLabelH(
	    (guint8 **)icon_convert_20x20_xpm,
	    "Convert",
	    NULL
	);
	gtk_widget_set_usize(
	    w, GUI_BUTTON_HLABEL_WIDTH, GUI_BUTTON_HLABEL_HEIGHT
	);
	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(WinConvertCB), win
	);
	GUISetWidgetTip(w,
	    "Convert the image(s)"
	);
	gtk_widget_show(w);

	/* Browse Button */
	win->imbr_btn = w = GUIButtonPixmapLabelH(
	    (guint8 **)icon_image_browser_20x20_xpm,
	    "Browse",
	    NULL      
	);
	gtk_widget_set_usize(
	    w, GUI_BUTTON_HLABEL_WIDTH, GUI_BUTTON_HLABEL_HEIGHT
	);
	gtk_box_pack_start(GTK_BOX(parent5), w, FALSE, FALSE, 0);
	gtk_signal_connect(
	    GTK_OBJECT(w), "clicked",
	    GTK_SIGNAL_FUNC(WinNewImbrCB), win
	);
	GUISetWidgetTip(w,
	    "Create a new image browser"
	);
	gtk_widget_show(w);


	/* Animated Icon */
	w = gtk_vbox_new(FALSE, border_major);
	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 0);
	gtk_widget_show(w);
	parent4 = w;

	w = gtk_frame_new(NULL);
	gtk_box_pack_start(GTK_BOX(parent4), w, FALSE, FALSE, 0);
	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
	gtk_widget_show(w);
	parent5 = w;

	win->anim_icon = w = WinAnimIconNew();
	gtk_container_add(GTK_CONTAINER(parent5), w);
	gtk_widget_show(w);



	/* Right side of paned */

	w = gtk_vbox_new(FALSE, border_major);
	gtk_paned_add2(GTK_PANED(parent), w);
	gtk_widget_show(w);
	parent2 = w;

	/* GtkScrolledWindow for the GtkCList */
	w = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(
	    GTK_SCROLLED_WINDOW(w),
	    GTK_POLICY_AUTOMATIC,
	    GTK_POLICY_AUTOMATIC
	);
	gtk_box_pack_start(GTK_BOX(parent2), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent3 = w;

	/* Source GtkCList */
	strv = (gchar **)g_malloc(columns * sizeof(gchar *));
	for(i = 0; i < columns; i++)
	    strv[i] = "";
	win->tar_clist = w = gtk_clist_new_with_titles(
	    columns, strv
	);
	g_free(strv);
	clist = GTK_CLIST(w);
	gtk_widget_add_events(
	    w,
	    GDK_FOCUS_CHANGE_MASK |
	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
	    GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK |
	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
	    GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_press_event",
	    GTK_SIGNAL_FUNC(WinCListKeyEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_release_event",
	    GTK_SIGNAL_FUNC(WinCListKeyEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_press_event",
	    GTK_SIGNAL_FUNC(WinCListButtonEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_release_event",
	    GTK_SIGNAL_FUNC(WinCListButtonEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "motion_notify_event",
	    GTK_SIGNAL_FUNC(WinCListMotionEventCB), core
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "focus_in_event",
	    GTK_SIGNAL_FUNC(WinTarListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "focus_out_event",
	    GTK_SIGNAL_FUNC(WinTarListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_press_event",
	    GTK_SIGNAL_FUNC(WinTarListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "key_release_event",
	    GTK_SIGNAL_FUNC(WinTarListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_press_event",
	    GTK_SIGNAL_FUNC(WinTarListEventCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_release_event",
	    GTK_SIGNAL_FUNC(WinTarListEventCB), win
	);
	gtk_container_add(GTK_CONTAINER(parent3), w);
	gtk_widget_realize(w);
	gtk_clist_set_selection_mode(clist, GTK_SELECTION_EXTENDED);
	for(i = 0; i < columns; i++)
	    gtk_clist_set_column_width(clist, i, 10);
	gtk_clist_column_titles_passive(clist);
	gtk_clist_set_column_title(clist, 0, "Converted Images");
	gtk_clist_set_row_height(clist, WIN_LIST_ROW_SPACING);
	gtk_clist_set_shadow_type(clist, GTK_SHADOW_IN);
	gtk_signal_connect(
	    GTK_OBJECT(w), "resize_column",
	    GTK_SIGNAL_FUNC(WinTarListResizeColumnCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "click_column",
	    GTK_SIGNAL_FUNC(WinTarListClickColumnCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "select_row",
	    GTK_SIGNAL_FUNC(WinTarListSelectRowCB), win
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "unselect_row",
	    GTK_SIGNAL_FUNC(WinTarListUnselectRowCB), win
	);
	GUIDNDSetSrc(
	    w,
	    dnd_src_types,
	    sizeof(dnd_src_types) / sizeof(GtkTargetEntry),
	    GDK_ACTION_MOVE | GDK_ACTION_COPY |	/* Actions */
		GDK_ACTION_LINK,
	    GDK_BUTTON1_MASK,			/* Buttons */
	    NULL,
	    WinCListDragDataGetCB,
	    WinCListDragDataDeleteCB,
	    NULL,
	    win
	); 
	GUIDNDSetTar(
	    w,
	    dnd_tar_types,
	    sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
	    GDK_ACTION_MOVE,			/* Actions */ 
	    GDK_ACTION_MOVE,			/* Default action if same */
	    GDK_ACTION_MOVE,			/* Default action */
	    WinCListDragDataReceivedCB,
	    win,
	    TRUE				/* Highlight */
	); 
	gtk_widget_show(w);


	/* Preview */
	win->tar_preview = w = WinPreviewNew();
	gtk_box_pack_start(GTK_BOX(parent2), w, FALSE, FALSE, 0);
	gtk_widget_realize(w);
	WinPreviewSetSize(w, -1, 100);
	WinPreviewSetShowDetails(w, win->preview_tar_show_details);
	GUIDNDSetTar(
	    WinPreviewGetDrawingArea(w),
	    dnd_tar_types,
	    sizeof(dnd_tar_types) / sizeof(GtkTargetEntry),
	    GDK_ACTION_MOVE,			/* Actions */
	    GDK_ACTION_MOVE,			/* Default action if same */
	    GDK_ACTION_MOVE,			/* Default action */
	    WinPreviewDragDataReceivedCB,
	    win,
	    TRUE				/* Highlight */
	);
	if(win->preview_tar)
	    gtk_widget_show(w);


}

/*
 *	Builds the Win's Status Bar.
 */
static void WinBuildStatusBar(win_struct *win)
{
	gint attach_x = 0;
	GtkAdjustment *adj;
	GtkWidget *w, *parent, *parent2, *parent3;

	parent = win->main_vbox;


	/* Begin recreating status bar */
	GTK_WIDGET_DESTROY(win->statusbar)

	/* Main outer frame */
	win->statusbar = w = gtk_frame_new(NULL);
	gtk_widget_set_usize(w, -1, WIN_STATUS_BAR_HEIGHT);
	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_OUT);
	gtk_box_pack_end(GTK_BOX(parent), w, FALSE, FALSE, 0);
	parent = w;

	/* Main table */
	w = gtk_table_new(1, 2, FALSE);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);
	parent = w;

	/* Progress bar */
	adj = (GtkAdjustment *)gtk_adjustment_new(
	    0.0f, 1.0f, 100.0f,
	    0.0f, 0.0f, 0.0f
	);
	win->statusbar_progress = w = gtk_progress_bar_new_with_adjustment(adj);
	gtk_widget_set_usize(w, 100, 20);
	gtk_progress_bar_set_orientation(
	    GTK_PROGRESS_BAR(w), GTK_PROGRESS_LEFT_TO_RIGHT
	);
	gtk_progress_bar_set_bar_style(
	    GTK_PROGRESS_BAR(w), GTK_PROGRESS_CONTINUOUS
	);
	gtk_progress_set_activity_mode(
	    GTK_PROGRESS(w), FALSE
	);
	gtk_table_attach(
	    GTK_TABLE(parent), w,
	    attach_x, attach_x + 1,
	    0, 1,
	    0,
	    GTK_SHRINK,
	    0, 0
	);
	attach_x++;
	gtk_widget_show(w);


	/* Label */
	w = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
	gtk_container_border_width(GTK_CONTAINER(w), 1);
	gtk_table_attach(
	    GTK_TABLE(parent), w,
	    attach_x, attach_x + 1,
	    0, 1,
	    GTK_SHRINK | GTK_EXPAND | GTK_FILL,
	    GTK_FILL,
	    0, 0
	);
	attach_x++;
	gtk_widget_show(w);
	parent2 = w;

	w = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(parent2), w);
	gtk_widget_show(w);
	parent3 = w;

	win->statusbar_label = w = gtk_label_new("");
	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(parent3), w, FALSE, FALSE, 2);
	gtk_widget_show(w);
}


/*
 *	Builds the Win's right click menu.
 */
static void WinBuildListsMenu(win_struct *win)
{
	GtkAccelGroup *accelgrp = win->accelgrp;
	GtkWidget *w, *menu;

#define ADD_MENU_ITEM_LABEL(_id_)	{	\
 win_opid_struct *opid = WinOPIDListFind(	\
  win->opid, (_id_)				\
 );						\
 if(opid != NULL) {				\
  w = GUIMenuItemCreate(			\
   menu,					\
   GUI_MENU_ITEM_TYPE_LABEL,			\
   accelgrp,					\
   opid->menu_icon_data, opid->menu_name,	\
   opid->accel_key, opid->accel_mods,		\
   opid->func_cb, opid->win			\
  );						\
  GUISetMenuItemCrossingCB(			\
   w,						\
   (gpointer)opid->enter_func_cb, opid,		\
   (gpointer)opid->leave_func_cb, opid		\
  );						\
 }						\
 else						\
  w = NULL;					\
}

#define ADD_MENU_ITEM_CHECK(_id_)	{	\
 win_opid_struct *opid = WinOPIDListFind(	\
  win->opid, (_id_)				\
 );						\
 if(opid != NULL) {				\
  w = GUIMenuItemCreate(			\
   menu, GUI_MENU_ITEM_TYPE_CHECK, accelgrp,	\
   opid->menu_icon_data, opid->menu_name,	\
   opid->accel_key, opid->accel_mods, NULL,	\
   opid->func_cb, opid->win			\
  );						\
  GUISetMenuItemCrossingCB(			\
   w,						\
   (gpointer)opid->enter_func_cb, opid,		\
   (gpointer)opid->leave_func_cb, opid		\
  );						\
 }						\
 else						\
  w = NULL;					\
}

#define ADD_MENU_ITEM_SEP	{		\
 w = GUIMenuItemCreate(				\
  menu,						\
  GUI_MENU_ITEM_TYPE_SEPARATOR,			\
  NULL,						\
  NULL,						\
  NULL,						\
  0, 0,						\
  NULL, NULL					\
 );						\
}

	/* Recreate the Source List's right click menu */
	GTK_WIDGET_DESTROY(win->src_list_menu);
	win->src_list_menu = menu = GUIMenuCreate();

	ADD_MENU_ITEM_LABEL(WIN_OPID_CONVERT);
	win->src_list_convert_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_CONVERT_SELECTED);
	win->src_list_convert_selected_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_OPEN);
	win->src_list_open_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_NEW_IMBR);
	win->src_list_new_imbr_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_ADD);
	win->src_list_add_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_REMOVE);
	win->src_list_remove_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_SHIFT_UP);
	win->src_list_shift_up_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_SHIFT_DOWN);
	win->src_list_shift_down_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_CUT);
	win->src_list_cut_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_COPY);
	win->src_list_copy_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_PASTE);
	win->src_list_paste_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_DELETE);
	win->src_list_delete_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_CLEAR_SRC);
	win->src_list_clear_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_SELECT_ALL);
	win->src_list_select_all_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_UNSELECT_ALL);
	win->src_list_unselect_all_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_INVERT_SELECTION);
	win->src_list_invert_selection_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_PROPERTIES);
	win->src_list_properties_mi = w;


	/* Recreate the Converted List's right click menu */
	GTK_WIDGET_DESTROY(win->tar_list_menu);
	win->tar_list_menu = menu = GUIMenuCreate();

	ADD_MENU_ITEM_LABEL(WIN_OPID_OPEN);
	win->tar_list_open_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_NEW_IMBR);
	win->tar_list_new_imbr_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_CUT);
	win->tar_list_cut_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_COPY);
	win->tar_list_copy_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_PASTE);
	win->tar_list_paste_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_DELETE);
	win->tar_list_delete_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_CLEAR_TAR);
	win->tar_list_clear_mi = w;

	ADD_MENU_ITEM_SEP

	ADD_MENU_ITEM_LABEL(WIN_OPID_SELECT_ALL);
	win->tar_list_select_all_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_UNSELECT_ALL);
	win->tar_list_unselect_all_mi = w;
	ADD_MENU_ITEM_LABEL(WIN_OPID_INVERT_SELECTION);
	win->tar_list_invert_selection_mi = w;

	ADD_MENU_ITEM_SEP
			 
	ADD_MENU_ITEM_LABEL(WIN_OPID_PROPERTIES);
	win->tar_list_properties_mi = w;

#undef ADD_MENU_ITEM_LABEL
#undef ADD_MENU_ITEM_CHECK
#undef ADD_MENU_ITEM_SEP
}



/*
 *	Creates a new Win.
 */
win_struct *WinNew(gpointer core_ptr)
{
	GdkWindow *window;
	GtkAccelGroup *accelgrp;
	GtkWidget *w, *parent, *parent2;
	GtkWidget *toplevel, *main_vbox;
	const CfgItem *cfg_list;
	core_struct *core = CORE(core_ptr);
	win_struct *win = WIN(
	    g_malloc0(sizeof(win_struct))
	);

	cfg_list = core->cfg_list;

	win->toplevel = toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	win->accelgrp = accelgrp = gtk_accel_group_new();
	win->processing = FALSE;
	win->busy_count = 0;
	win->passive_busy_count = 0;
	win->freeze_count = 0;
	win->stop_count = 0;
	win->core = core_ptr;
	win->opid = WinOPIDListNew(win);
	win->menubar_map_state = TRUE;
	win->tool_bar_map_state = TRUE;
	win->focus_clist = NULL;
	win->convparm_list = NULL;
	win->statusbar_map_state = TRUE;
	win->statusbar_progress_last = 0.0f;
	win->preview_src = CFG_GET_B(CFG_PARM_WIN_PREVIEW_SRC);
	win->preview_src_show_details = CFG_GET_B(
	    CFG_PARM_WIN_PREVIEW_SRC_SHOW_DETAILS
	);
	win->preview_tar = CFG_GET_B(CFG_PARM_WIN_PREVIEW_TAR);
	win->preview_tar_show_details = CFG_GET_B(
	    CFG_PARM_WIN_PREVIEW_TAR_SHOW_DETAILS  
	);
	win->tar_dir = NULL;
	win->tar_name_prefix = NULL;
	win->tar_name_postfix = NULL;
	win->remove_converted = TRUE;
	win->recursive = FALSE;

	/* Toplevel GtkWindow */
	w = toplevel;
	gtk_window_set_wmclass(
	    GTK_WINDOW(w), "window", PROG_NAME
	);
	gtk_window_set_policy(GTK_WINDOW(w), TRUE, TRUE, FALSE);
	gtk_window_set_title(GTK_WINDOW(w), PROG_NAME);
	if(cfg_list != NULL)
	{
	    const gint	x = CFG_GET_I(CFG_PARM_WIN_X),
			y = CFG_GET_I(CFG_PARM_WIN_Y),
			width = CFG_GET_I(CFG_PARM_WIN_WIDTH),
			height = CFG_GET_I(CFG_PARM_WIN_HEIGHT);
	    if((width > 0) && (height > 0))
	    {
		gtk_widget_set_usize(w, width, height);

		/* Set the position only if this is the first window */
		if(g_list_length(core->win) == 0)
		    gtk_widget_set_uposition(w, x, y);
	    }
	    else
	    {
		gtk_widget_set_usize(w, WIN_WIDTH, WIN_HEIGHT);
	    }
	}
	gtk_widget_realize(w);
	window = w->window;
	if(window != NULL)
	{
#if 0
	    gdk_window_set_decorations(
		window,
		GDK_DECOR_TITLE | GDK_DECOR_MENU | GDK_DECOR_MINIMIZE
	    );
	    gdk_window_set_functions(
		window,
		GDK_FUNC_MOVE | GDK_FUNC_MINIMIZE | GDK_FUNC_CLOSE
	    );
#endif
	    GUISetWMIcon(
		window,
		(guint8 **)icon_imgworks_48x48_xpm
	    );
	}
	gtk_signal_connect(
	    GTK_OBJECT(w), "delete_event",
	    GTK_SIGNAL_FUNC(WinDeleteEventCB), win
	);
	gtk_container_border_width(GTK_CONTAINER(w), 0);
	gtk_window_add_accel_group(GTK_WINDOW(w), accelgrp);
	parent = w;


	/* Main VBox */
	win->main_vbox = main_vbox = w = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(w), 0);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);
	parent = w;


	/* Build Menu Bar */
	win->menubar_handle = w = gtk_handle_box_new();
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	win->menubar = NULL;
	WinBuildMenuBar(win);

	/* Build Tool Bar */
	win->tool_bar_handle = w = gtk_handle_box_new();
	gtk_box_pack_start(GTK_BOX(parent), w, FALSE, FALSE, 0);
	win->tool_bar = NULL;
	WinBuildToolBar(win);


	/* GtkPaned for the lists */
	win->lists_paned1 = w = gtk_hpaned_new();
	if(cfg_list != NULL)
	{
	    gint x = CFG_GET_I(CFG_PARM_WIN_PANED1_POSITION);
	    if(x <= 0)
		x = WIN_PANED1_POSITION;
	    gtk_paned_set_position(GTK_PANED(w), x);
	}
	gtk_paned_set_handle_size(GTK_PANED(w), 10);
	gtk_paned_set_gutter_size(GTK_PANED(w), 10);
	gtk_box_pack_start(GTK_BOX(parent), w, TRUE, TRUE, 0);
	gtk_widget_show(w);
	parent2 = w;

	/* Build the lists */
	WinBuildLists(win);


	/* Build the Status Bar */
	win->statusbar = NULL;
	WinBuildStatusBar(win);


	/* Build the right-click popup menus */
	win->src_list_menu = NULL;
	win->tar_list_menu = NULL;
	WinBuildListsMenu(win);


	/* Show/hide menu bar */
	w = win->menubar_handle;
	if(win->menubar_map_state)
	    gtk_widget_show(w);
	else
	    gtk_widget_hide(w);

	/* Show/hide the tool bar */
	w = win->tool_bar_handle;
	if(win->tool_bar_map_state)
	    gtk_widget_show(w);
	else
	    gtk_widget_hide(w);

	/* Show/hide the status bar */
	w = win->statusbar;
	if(win->statusbar_map_state)
	    gtk_widget_show(w);
	else
	    gtk_widget_hide(w);



	/* Set the initial convert options */
	if(cfg_list != NULL)
	{
	    WinSetTarDir(win, CFG_GET_S(CFG_PARM_TAR_DIR));
	    WinSetRemoveConverted(win, CFG_GET_B(CFG_PARM_REMOVE_CONVERTED));
	    WinSetRecursive(win, CFG_GET_B(CFG_PARM_RECURSIVE));

	}

	/* Set the source clist as the focus clist */
	win->focus_clist = win->src_clist;


	WinUpdate(win);


	return(win);
}

/*
 *	Records the Win's widget positions to the configuration.
 */
void WinSavePositions(win_struct *win)
{
	GtkWidget *w;
	core_struct *core = CORE((win != NULL) ? win->core : NULL);
	CfgItem *cfg_list = (core != NULL) ? core->cfg_list : NULL;

	if(cfg_list == NULL)
	    return;

	/* Convert Options */
	CFG_SET_S(CFG_PARM_TAR_DIR, WinGetTarDir(win));
	CFG_SET_B(CFG_PARM_REMOVE_CONVERTED, WinGetRemoveConverted(win));
	CFG_SET_B(CFG_PARM_RECURSIVE, WinGetRecursive(win));


	/* Toplevel */
	w = win->toplevel;
	if(w != NULL)
	{
	    GdkWindow *window = w->window;
	    if(window != NULL)
	    {
		gint x, y;
		gdk_window_get_root_origin(window, &x, &y);
		CFG_SET_I(CFG_PARM_WIN_X, x);
		CFG_SET_I(CFG_PARM_WIN_Y, y);
	    }
	    CFG_SET_I(CFG_PARM_WIN_WIDTH, w->allocation.width);
	    CFG_SET_I(CFG_PARM_WIN_HEIGHT, w->allocation.height);
	}

	/* Paned */
	w = win->lists_paned1;
	if(w != NULL)
	    CFG_SET_I(
		CFG_PARM_WIN_PANED1_POSITION,
		GTK_PANED(w)->child1_size
	    );
	w = win->lists_paned2;
	if(w != NULL)
	    CFG_SET_I(
		CFG_PARM_WIN_PANED2_POSITION,
		GTK_PANED(w)->child1_size
	    );

	/* Show/hide */
	CFG_SET_B(CFG_PARM_WIN_PREVIEW_SRC, win->preview_src);
        CFG_SET_B(
	    CFG_PARM_WIN_PREVIEW_SRC_SHOW_DETAILS,
	    win->preview_src_show_details
	);
	CFG_SET_B(CFG_PARM_WIN_PREVIEW_TAR, win->preview_tar);
        CFG_SET_B(
	    CFG_PARM_WIN_PREVIEW_TAR_SHOW_DETAILS,
	    win->preview_tar_show_details
	);

}

/*
 *	Updates the Win's widgets to reflect current values.
 */
void WinUpdate(win_struct *win)
{
	gboolean sensitive, toggled, processing;
	gint	row = -1, nrows = 0,
		src_row = -1, src_nrows = 0,
		tar_row = -1, tar_nrows = 0;
	GtkCList *clist;
	GtkWidget *tool_bar;

	if(win == NULL)
	    return;

	if(win->freeze_count > 0)
	    return;

	processing = win->processing;
	tool_bar = win->tool_bar;

	clist = (GtkCList *)win->focus_clist;
	if(clist != NULL)
	{
	    GList *glist = clist->selection_end;
	    if(glist != NULL)
		row = (gint)glist->data;
	    nrows = clist->rows;
	}
	clist = (GtkCList *)win->src_clist;
	if(clist != NULL)
	{
	    GList *glist = clist->selection_end;
	    if(glist != NULL)
		src_row = (gint)glist->data;
	    src_nrows = clist->rows;
	}
	clist = (GtkCList *)win->tar_clist;
	if(clist != NULL)
	{
	    GList *glist = clist->selection_end;
	    if(glist != NULL)
		tar_row = (gint)glist->data;
	    tar_nrows = clist->rows;
	}

	win->freeze_count++;

	/* Close */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->close_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_CLOSE, sensitive
	);

	/* New */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->new_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_NEW, sensitive
	);
	/* Add */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->file_add_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->add_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_ADD, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_convert_mi, sensitive);
	/* Remove */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->file_remove_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->remove_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_REMOVE, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_remove_mi, sensitive);
	/* Shift Up */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->shift_up_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_SHIFT_UP, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_shift_up_mi, sensitive);
	/* Shift Down */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->shift_down_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_SHIFT_DOWN, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_shift_down_mi, sensitive);

	/* Cut */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->cut_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_CUT, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_cut_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_cut_mi, sensitive);
	/* Copy */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->copy_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_COPY, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_copy_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_copy_mi, sensitive);
	/* Paste */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->paste_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_PASTE, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_paste_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_paste_mi, sensitive);
	/* Delete */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->delete_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_DELETE, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_delete_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_delete_mi, sensitive);
	/* Clear Source */
	sensitive = (!processing && (src_nrows > 0)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->clear_src_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_CLEAR_SRC, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_clear_mi, sensitive);
	/* Clear Converted */
	sensitive = (!processing && (tar_nrows > 0)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->clear_tar_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_CLEAR_TAR, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_clear_mi, sensitive);

	/* Select All */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->select_all_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_SELECT_ALL, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_select_all_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_select_all_mi, sensitive);
	/* Unselect All */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->unselect_all_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_UNSELECT_ALL, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_unselect_all_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_unselect_all_mi, sensitive);
	/* Invert Selection */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->invert_selection_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_INVERT_SELECTION, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_invert_selection_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_invert_selection_mi, sensitive);

	/* Properties */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->properties_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_PROPERTIES, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_properties_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_properties_mi, sensitive);

	/* Open */
	sensitive = (!processing && (row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->open_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_OPEN, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_open_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_open_mi, sensitive);
	/* Refresh */
	sensitive = TRUE;
	GTK_WIDGET_SET_SENSITIVE(win->refresh_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_REFRESH, sensitive
	);
	/* Preview Source */
	toggled = win->preview_src;
	GUIMenuItemSetCheck(
	    win->preview_src_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_PREVIEW_SRC, toggled
	);
	/* Preview Source Show Details*/
	toggled = win->preview_src_show_details;
	GUIMenuItemSetCheck(
	    win->preview_src_show_details_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_PREVIEW_SRC_SHOW_DETAILS, toggled
	);
	/* Preview Converted */
	toggled = win->preview_tar;
	GUIMenuItemSetCheck(
	    win->preview_tar_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_PREVIEW_TAR, toggled
	);
	/* Preview Converted Show Details */
	toggled = win->preview_tar_show_details;
	GUIMenuItemSetCheck(
	    win->preview_tar_show_details_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_PREVIEW_TAR_SHOW_DETAILS, toggled
	);
	/* View Tool Bar */
	toggled = win->tool_bar_map_state;
	GUIMenuItemSetCheck(
	    win->view_tool_bar_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_VIEW_TOOLBAR, toggled
	);
	/* View Status Bar */
	toggled = win->statusbar_map_state;
	GUIMenuItemSetCheck(
	    win->view_statusbar_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_VIEW_STATUSBAR, toggled
	);



	/* Convert */
	sensitive = (!processing && (src_nrows > 0)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->convert_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->convert_btn, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_CONVERT, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_convert_mi, sensitive);
	/* Convert Selected */
	sensitive = (!processing && (src_row > -1)) ? TRUE : FALSE;
	GTK_WIDGET_SET_SENSITIVE(win->convert_selected_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_CONVERT_SELECTED, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_convert_selected_mi, sensitive);
	/* Stop */
	sensitive = processing;
	GTK_WIDGET_SET_SENSITIVE(win->stop_mi, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_STOP, sensitive
	);
	/* Remove Converted */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->remove_converted_micheck, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_REMOVE_CONVERTED, sensitive
	);
	toggled = win->remove_converted;
	GUIMenuItemSetCheck(
	    win->remove_converted_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_REMOVE_CONVERTED, toggled
	);
	/* Recursive */
	sensitive = !processing;
	GTK_WIDGET_SET_SENSITIVE(win->recursive_micheck, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_RECURSIVE, sensitive
	);
	toggled = win->recursive;
	GUIMenuItemSetCheck(
	    win->recursive_micheck, toggled, TRUE
	);
	ToolBarSetItemToggle(
	    tool_bar, WIN_OPID_RECURSIVE, toggled
	);

	/* ImgWorks */
	sensitive = TRUE;
        GTK_WIDGET_SET_SENSITIVE(win->new_imgworks_mi, sensitive);

	/* Image Browser */
	sensitive = TRUE;
	GTK_WIDGET_SET_SENSITIVE(win->new_imbr_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->imbr_btn, sensitive);
	ToolBarSetItemSensitive(
	    tool_bar, WIN_OPID_NEW_IMBR, sensitive
	);
	GTK_WIDGET_SET_SENSITIVE(win->src_list_new_imbr_mi, sensitive);
	GTK_WIDGET_SET_SENSITIVE(win->tar_list_new_imbr_mi, sensitive);


	win->freeze_count--;
}

/*
 *	Sets the Win as busy or ready.
 */
void WinSetBusy(win_struct *win, gboolean busy)
{
	GdkCursor *cur;
	GtkWidget *w;
	core_struct *core;
	if(win == NULL)
	    return;

	w = win->toplevel;
	core = win->core;
	if((w == NULL) || (core == NULL))
	    return;

	if(busy)
	{
	    win->busy_count++;
	    if(win->busy_count > 1)
		return;
	    cur = core->busy_cur;
	}
	else
	{
	    win->busy_count--;
	    if(win->busy_count < 0)
		win->busy_count = 0;
	    if(win->busy_count > 0)
		return;
	    cur = (win->passive_busy_count > 0) ?
		core->passive_busy_cur : NULL;
	}
	 
	if(w->window != NULL)
	{
	    gdk_window_set_cursor(w->window, cur);
	    gdk_flush();
	}
}

/*
 *	Sets the Win as passive busy or ready.
 */
void WinSetPassiveBusy(win_struct *win, gboolean busy)
{
	GdkCursor *cur;
	GtkWidget *w;
	core_struct *core;
 
	if(win == NULL)
	    return;

	w = win->toplevel;
	core = CORE(win->core);
	if((w == NULL) || (core == NULL))
	    return;

	if(busy)
	{
	    win->passive_busy_count++;
	    if((win->passive_busy_count > 1) || (win->busy_count > 0))
		return;
	    cur = core->passive_busy_cur;
	}
	else
	{
	    win->passive_busy_count--;
	    if(win->passive_busy_count < 0)
		win->passive_busy_count = 0;
	    if((win->passive_busy_count > 0) || (win->busy_count > 0))
		return;
	    cur = NULL;
	}

	if(w->window != NULL)
	{
	    gdk_window_set_cursor(w->window, cur);
	    gdk_flush();
	}
}

/*
 *	Checks if the Win's Tool Bar is shown.      
 */
gboolean WinToolBarIsShown(win_struct *win)
{
	return((win != NULL) ? win->tool_bar_map_state : FALSE);
}

/*
 *	Show/hide the Win's Tool Bar.
 */
void WinToolBarSetShow(win_struct *win, gboolean show)
{
	GtkWidget *w = (win != NULL) ? win->tool_bar_handle : NULL;
	if(w == NULL)
	    return;

	win->freeze_count++;

	if(win->tool_bar_map_state && !show)
	{
	    gtk_widget_hide(w);
	    win->tool_bar_map_state = FALSE;
	}
	else if(!win->tool_bar_map_state && show)
	{
	    gtk_widget_show(w);
	    win->tool_bar_map_state = TRUE;
	}

	WinUpdate(win);

	win->freeze_count--;
}

/*
 *	Recreates the Win's Tool Bar Items.
 */
void WinToolBarUpdateItems(win_struct *win)
{
	if(win == NULL)
	    return;

	/* Recreate Tool Bar and the Tool Bar Items */
	WinBuildToolBar(win);
}


/*
 *	Checks if the Win's Status Bar is shown.
 */
gboolean WinStatusBarIsShown(win_struct *win)
{
	return((win != NULL) ? win->statusbar_map_state : FALSE);
}

/*
 *	Show/hide the Win's Status Bar.
 */
void WinStatusBarSetShow(win_struct *win, gboolean show)
{
	GtkWidget *w = (win != NULL) ? win->statusbar : NULL;
	if(w == NULL)
	    return;

	win->freeze_count++;

	if(win->statusbar_map_state && !show)
	{
	    gtk_widget_hide(w);
	    win->statusbar_map_state = FALSE;
	}
	else if(!win->statusbar_map_state&& show)
	{
	    gtk_widget_show(w);
	    win->statusbar_map_state = TRUE;
	}

	WinUpdate(win);

	win->freeze_count--;
}


/*
 *	Sets the Win's status bar progress.
 */
void WinStatusProgress(
	win_struct *win, gfloat val, gboolean allow_gtk_iteration
)
{
	GtkProgress *progress;
	GtkWidget *w = (win != NULL) ? win->statusbar_progress : NULL;
	if(w == NULL)
	    return;

	progress = GTK_PROGRESS(w);

	/* Do activity? */
	if(val < 0.0f)
	{
	    GtkAdjustment *adj = progress->adjustment;

	    /* Get new value based on unknown position */
	    val = gtk_progress_get_value(progress) + 1.0f;
	    if(val > adj->upper)
		val = adj->lower;

	    /* Update progress bar for `activity mode' (unknown limit)
	     * and set new value            
	     */
	    gtk_progress_set_activity_mode(progress, TRUE);
	    gtk_progress_set_show_text(progress, FALSE);
	    gtk_progress_set_value(progress, val);

	    /* Reset last progress position to -1.0, indicating that
	     * there is no known last position since we are doing
	     * activity mode
	     */
	    win->statusbar_progress_last = -1.0f;
	}
	else
	{
	    /* Clip value to 0.0, 1.0 */
	    if(val > 1.0f)
		val = 1.0f;

	    /* Reset last progress position if it is greater than the
	     * given value, implying the progress has warped back to
	     * the beginning
	     */
	    if(win->statusbar_progress_last > val)
		win->statusbar_progress_last = -1.0f;

	    /* Has value not sufficiently changed? (less than 0.01
	     * change in value)
	     */
	    if((win->statusbar_progress_last > 0.0f) &&
	       ((val - win->statusbar_progress_last) < 0.01f)
	    )
		return;

	    /* Update progress */
	    gtk_progress_set_activity_mode(progress, FALSE);
	    gtk_progress_set_format_string(
		progress, "%p%%"
	    );
	    /* Display text only if value is positive */
	    gtk_progress_set_show_text(
		progress, (val > 0.0f) ? TRUE : FALSE 
	    );
	    gtk_progress_bar_update(GTK_PROGRESS_BAR(w), val); 

	    /* Record last position */
	    win->statusbar_progress_last = val;
	}

	/* Process events? */
	if(allow_gtk_iteration)
	    gtk_events_process();
}

/*
 *	Sets the Win's status bar message.
 */
void WinStatusMessage(
	win_struct *win, const gchar *mesg, gboolean allow_gtk_iteration
)
{
	GtkWidget *w = (win != NULL) ? win->statusbar_label : NULL;
	if(w == NULL)
	    return;

	gtk_label_set_text(
	    GTK_LABEL(w), (mesg != NULL) ? mesg : ""
	);

	if(allow_gtk_iteration)
	    gtk_events_process();
}

/*
 *	Checks if the Win is mapped.
 */
gboolean WinIsMapped(win_struct *win)
{
	GtkWidget *w = (win != NULL) ? win->toplevel : NULL;
	return((w != NULL) ? GTK_WIDGET_MAPPED(w) : FALSE);
}

/*
 *	Maps the Win.
 */
void WinMap(win_struct *win)
{
	GtkWidget *w = (win != NULL) ? win->toplevel : NULL;
	if(w == NULL)
	    return;

	gtk_widget_show_raise(w);
}

/*
 *	Unmaps the Win.
 */
void WinUnmap(win_struct *win)
{
	GtkWidget *w = (win != NULL) ? win->toplevel : NULL;
	if(w == NULL)
	    return;

	gtk_widget_hide(w);
}

/*
 *	Deletes the Win.
 */
void WinDelete(win_struct *win)
{
	if(win == NULL)
	    return;

	WinSrcListClear(win);
	WinTarListClear(win);

	win->freeze_count++;

	GTK_WIDGET_DESTROY(win->src_list_menu);
	win->src_list_menu = NULL;
	GTK_WIDGET_DESTROY(win->tar_list_menu);
	win->tar_list_menu = NULL;

	GTK_WIDGET_DESTROY(win->menubar);
	GTK_WIDGET_DESTROY(win->menubar_handle);

	GTK_WIDGET_DESTROY(win->tool_bar);
	GTK_WIDGET_DESTROY(win->tool_bar_handle);

	GTK_WIDGET_DESTROY(win->anim_icon);
	GTK_WIDGET_DESTROY(win->imbr_btn);
	GTK_WIDGET_DESTROY(win->convert_btn);
	GTK_WIDGET_DESTROY(win->tar_dir_clear_btn);
	GTK_WIDGET_DESTROY(win->tar_dir_browse_btn);
	GTK_WIDGET_DESTROY(win->tar_dir_entry);

	g_list_foreach(
	    win->convparm_list, (GFunc)ConvParmDelete, NULL
	);
	g_list_free(win->convparm_list);

	GTK_WIDGET_DESTROY(win->tar_preview);
	GTK_WIDGET_DESTROY(win->tar_clist);
	GTK_WIDGET_DESTROY(win->src_preview);
	GTK_WIDGET_DESTROY(win->src_clist);
	GTK_WIDGET_DESTROY(win->lists_paned2);
	GTK_WIDGET_DESTROY(win->lists_paned1);

	GTK_WIDGET_DESTROY(win->statusbar);

	GTK_WIDGET_DESTROY(win->main_vbox);
	GTK_WIDGET_DESTROY(win->toplevel);

	GTK_ACCEL_GROUP_UNREF(win->accelgrp);

	WinOPIDListDelete(win->opid);

	g_free(win->tar_dir);
	g_free(win->tar_name_prefix);
	g_free(win->tar_name_postfix);

	win->freeze_count--;

	g_free(win);
}
