#include <sys/stat.h>
#include <gtk/gtk.h>
#ifdef HAVE_LIBENDEAVOUR2
# include <endeavour2.h>
#endif
#include "guiutils.h"
#include "obj.h"
#include "win.h"
#include "winlist.h"
#include "core.h"
#include "config.h"


static void WinListItemDestroyCB(gpointer data);

gint WinSrcListInsert(
	win_struct *win, gint row,
	obj_struct *obj			/* Transfered */
);
gint WinSrcListAppend(
	win_struct *win,
	obj_struct *obj			/* Transfered */
);
void WinSrcListRemove(win_struct *win, gint row);
void WinSrcListClear(win_struct *win);
void WinSrcListUpdateRow(
	win_struct *win, gint row, obj_struct *obj
);

gint WinTarListInsert(
	win_struct *win, gint row,
	obj_struct *obj			/* Transfered */
);
gint WinTarListAppend(
	win_struct *win,
	obj_struct *obj			/* Transfered */
);
void WinTarListRemove(win_struct *win, gint row);
void WinTarListClear(win_struct *win);
void WinTarListUpdateRow(
	win_struct *win, gint row, obj_struct *obj
);


#define ATOI(s)         (((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)         (((s) != NULL) ? atol(s) : 0)
#define ATOF(s)         (((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)       (((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)        (((a) > (b)) ? (a) : (b))
#define MIN(a,b)        (((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)     (MIN(MAX((a),(l)),(h)))
#define STRLEN(s)       (((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)   (((s) != NULL) ? (*(s) == '\0') : TRUE)


/*
 *      Win list item destroy signal callback.
 */
static void WinListItemDestroyCB(gpointer data)
{
	ObjDelete(OBJ(data));
}


/*
 *	Inserts the Object to the Win's List at the specified row.
 *
 *	The Object will be transfered and should not be referenced
 *	after this call.
 */
gint WinSrcListInsert(
	win_struct *win, gint row,
	obj_struct *obj                 /* Transfered */
)
{
	gint i, columns, new_row;
	gchar **strv;
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->src_clist : NULL;
	if(clist == NULL)
	{
	    ObjDelete(obj);
	    return(-1);
	}

	columns = clist->columns;
	if(columns <= 0)
	{
	    ObjDelete(obj);
	    return(-1);    
	}

	/* Allocate row cell values */
	strv = (gchar **)g_malloc(columns * sizeof(gchar *));
	for(i = 0; i < columns; i++)
	    strv[i] = "";

	gtk_clist_freeze(clist);

	/* Insert or append new row */
	if((row >= 0) && (row < clist->rows))
	    new_row = gtk_clist_insert(clist, row, strv);
	else
	    new_row = gtk_clist_append(clist, strv);

	/* Delete row cell values */
	g_free(strv);

	/* Failed to create new row? */
	if(new_row < 0)
	{
	    ObjDelete(obj);
	    gtk_clist_thaw(clist);
	    return(-1);    
	}

	/* Set the new row's data as the Object */
	gtk_clist_set_row_data_full(
	    clist, new_row,
	    obj, WinListItemDestroyCB
	);

	/* Update the row cell values with the new object */
	WinSrcListUpdateRow(win, new_row, obj);

	gtk_clist_thaw(clist);

	return(new_row);
}

/*
 *	Appends the Object to the Win's List.
 *
 *	The Object will be transfered and should not be referenced
 *	after this call.
 */
gint WinSrcListAppend(
	win_struct *win,
	obj_struct *obj                 /* Transfered */
)
{
	return(WinSrcListInsert(win, -1, obj));
}

/*
 *	Removes the specified row on the Win's List.
 */
void WinSrcListRemove(win_struct *win, gint row)
{
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->src_clist : NULL;
	if(clist == NULL)
	    return;

	gtk_clist_freeze(clist);
	gtk_clist_remove(clist, row);
	gtk_clist_thaw(clist);
}

/*
 *	Clears all rows on the Win's List.
 */
void WinSrcListClear(win_struct *win)
{
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->src_clist : NULL;
	if(clist == NULL)
	    return;

	gtk_clist_freeze(clist);
	gtk_clist_clear(clist);
	gtk_clist_thaw(clist);
}

/*
 *	Updates the specified row on the Win's List with the values
 *	specified in the Object.
 */
void WinSrcListUpdateRow(win_struct *win, gint row, obj_struct *obj)
{
	gint column, columns;
#ifdef HAVE_LIBENDEAVOUR2
	EDVContext *edv_ctx;
#endif
	core_struct *core;
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->src_clist : NULL;
	if(clist == NULL)
	    return;

	if((row < 0) || (row >= clist->rows))
	    return;

	core = CORE(win->core);
#ifdef HAVE_LIBENDEAVOUR2
	edv_ctx = (core != NULL) ? core->edv_ctx : NULL;
#endif

	columns = clist->columns;
	if(columns <= 0)
	    return;

	gtk_clist_freeze(clist);

	column = 0;
	if(column < columns)
	{
	    if(obj != NULL)
	    {
		struct stat lstat_buf;
		gchar	*name = obj->name,
			*full_path = obj->full_path;
		if(!lstat((const char *)full_path, &lstat_buf))
		{
		    GdkBitmap *mask = NULL;
		    GdkPixmap *pixmap = NULL;
#if defined(HAVE_LIBENDEAVOUR2)
		    gchar	*icon_path,
	                        *icon_opened_path,
	                        *icon_inaccessable_path,
	                        *icon_hidden_path;
		    EDVVFSObject *obj = edv_vfs_object_lstat(full_path);
		    (void)edv_match_vfs_object_icon(
			edv_ctx,
			obj,
			EDV_ICON_SIZE_20,
			&icon_path,
			&icon_opened_path,
			&icon_inaccessable_path,
			&icon_hidden_path
		    );
		    if(edv_path_is_hidden(full_path))
		    {
			if(icon_hidden_path != NULL)
			{
			    g_free(icon_path);
			    icon_path = g_strdup(icon_hidden_path);
			}
		    }
		    if(icon_path != NULL)
			pixmap = GDK_PIXMAP_NEW_FROM_XPM_FILE(
			    &mask,
			    icon_path
			);
		    edv_vfs_object_delete(obj);
		    g_free(icon_path);
		    g_free(icon_opened_path);
		    g_free(icon_inaccessable_path);
		    g_free(icon_hidden_path);
#endif
		    if(pixmap != NULL)
			gtk_clist_set_pixtext(
			    clist,
			    row, column,
			    name,
			    WIN_LIST_PIXMAP_TEXT_SPACING,
			    pixmap, mask
			);
		    else
			gtk_clist_set_text(
			    clist,
			    row, column,
			    name
			);

		    (void)GDK_PIXMAP_UNREF(pixmap);
		    (void)GDK_BITMAP_UNREF(mask);
		}
		else
		{
		    gtk_clist_set_text(clist, row, column, name);
		}
	    }
	    else
	    {
		gtk_clist_set_text(clist, row, column, "");
	    }
	}

	gtk_clist_thaw(clist);
}


/*
 *	Inserts the Object to the Win's List at the specified row.
 *
 *	The Object will be transfered and should not be referenced
 *	after this call.
 */
gint WinTarListInsert(
	win_struct *win, gint row,
	obj_struct *obj                 /* Transfered */
)
{
	gint i, columns, new_row;
	gchar **strv;
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->tar_clist : NULL;
	if(clist == NULL)
	{
	    ObjDelete(obj);
	    return(-1);
	}

	columns = clist->columns;
	if(columns <= 0)
	{
	    ObjDelete(obj);
	    return(-1);    
	}

	/* Allocate row cell values */
	strv = (gchar **)g_malloc(columns * sizeof(gchar *));
	for(i = 0; i < columns; i++)
	    strv[i] = "";

	gtk_clist_freeze(clist);

	/* Insert or append new row */
	if((row >= 0) && (row < clist->rows))
	    new_row = gtk_clist_insert(clist, row, strv);
	else
	    new_row = gtk_clist_append(clist, strv);

	/* Delete row cell values */
	g_free(strv);

	/* Failed to create new row? */
	if(new_row < 0)
	{
	    ObjDelete(obj);
	    gtk_clist_thaw(clist);
	    return(-1);    
	}

	/* Set the new row's data as the Object */
	gtk_clist_set_row_data_full(
	    clist, new_row,
	    obj, WinListItemDestroyCB
	);

	/* Update the row cell values with the new object */
	WinTarListUpdateRow(win, new_row, obj);

	gtk_clist_thaw(clist);

	return(new_row);
}

/*
 *	Appends the Object to the Win's List.
 *
 *	The Object will be transfered and should not be referenced
 *	after this call.
 */
gint WinTarListAppend(
	win_struct *win,
	obj_struct *obj                 /* Transfered */
)
{
	return(WinTarListInsert(win, -1, obj));
}

/*
 *	Removes the specified row on the Win's List.
 */
void WinTarListRemove(win_struct *win, gint row)
{
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->tar_clist : NULL;
	if(clist == NULL)
	    return;

	gtk_clist_freeze(clist);
	gtk_clist_remove(clist, row);
	gtk_clist_thaw(clist);
}

/*
 *	Clears all rows on the Win's List.
 */
void WinTarListClear(win_struct *win)
{
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->tar_clist : NULL;
	if(clist == NULL)
	    return;

	gtk_clist_freeze(clist);
	gtk_clist_clear(clist);
	gtk_clist_thaw(clist);
}

/*
 *	Updates the specified row on the Win's List with the values
 *	specified in the Object.
 */
void WinTarListUpdateRow(win_struct *win, gint row, obj_struct *obj)
{
	gint column, columns;
#ifdef HAVE_LIBENDEAVOUR2
	EDVContext *edv_ctx;
#endif
	core_struct *core;
	GtkCList *clist = (win != NULL) ?
	    (GtkCList *)win->tar_clist : NULL;
	if(clist == NULL)
	    return;

	if((row < 0) || (row >= clist->rows))
	    return;

	core = CORE(win->core);
#ifdef HAVE_LIBENDEAVOUR2
	edv_ctx = (core != NULL) ? core->edv_ctx : NULL;
#endif

	columns = clist->columns;
	if(columns <= 0)
	    return;

	gtk_clist_freeze(clist);

	column = 0;
	if(column < columns)
	{
	    if(obj != NULL)
	    {
		struct stat lstat_buf;
		gchar	*name = obj->name,
			*full_path = obj->full_path;
		if(!lstat((const char *)full_path, &lstat_buf))
		{
		    GdkBitmap *mask = NULL;
		    GdkPixmap *pixmap = NULL;
#if defined(HAVE_LIBENDEAVOUR2)
		    gchar	*icon_path,
	                        *icon_opened_path,
	                        *icon_inaccessable_path,
	                        *icon_hidden_path;
		    EDVVFSObject *obj = edv_vfs_object_lstat(full_path);
		    (void)edv_match_vfs_object_icon(
			edv_ctx,
			obj,
			EDV_ICON_SIZE_20,
			&icon_path,
			&icon_opened_path,
			&icon_inaccessable_path,
			&icon_hidden_path
		    );
		    if(edv_path_is_hidden(full_path))
		    {
			if(icon_hidden_path != NULL)
			{
			    g_free(icon_path);
			    icon_path = g_strdup(icon_hidden_path);
			}
		    }
		    if(icon_path != NULL)
			pixmap = GDK_PIXMAP_NEW_FROM_XPM_FILE(
			    &mask,
			    icon_path
			);
		    edv_vfs_object_delete(obj);
		    g_free(icon_path);
		    g_free(icon_opened_path);
		    g_free(icon_inaccessable_path);
		    g_free(icon_hidden_path);
#endif
		    if(pixmap != NULL)
			gtk_clist_set_pixtext(
			    clist,
			    row, column,
			    name,
			    WIN_LIST_PIXMAP_TEXT_SPACING,
			    pixmap, mask
			);
		    else
			gtk_clist_set_text(
			    clist,
			    row, column,
			    name
			);

		    (void)GDK_PIXMAP_UNREF(pixmap);
		    (void)GDK_BITMAP_UNREF(mask);
		}
		else
		{
		    gtk_clist_set_text(clist, row, column, name);
		}
	    }
	    else
	    {
		gtk_clist_set_text(clist, row, column, "");
	    }
	}

	gtk_clist_thaw(clist);
}
