#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef HAVE_IMAGE_MAGICK
# include <wand/magick_wand.h>
#endif
#include <gtk/gtk.h>
#include <gdk/gdkrgb.h>
#ifdef HAVE_LIBENDEAVOUR2
# include <endeavour2.h>
#endif

#include "guiutils.h"

#include "winpreview.h"


/*
 *	Preview:
 */
typedef struct {

	GtkWidget	*toplevel,
			*drawing_area;

	GdkPixmap	*pixmap;

	GList		*img_pixmap_list;	/* GdkPixmap * */
	GList		*img_mask_list;		/* GdkBitmap * */
	GList		*img_delay_list;	/* guint in ms */
	gint		frame_num;
	gboolean	show_details;
	gchar		*img_label;
	gchar		*img_file;		/* Full path to image file */

	guint		draw_next_frame_toid;

#ifdef HAVE_LIBENDEAVOUR2
	EDVContext	*edv_ctx;
#endif

} WinPreview;
#define WIN_PREVIEW(p)		((WinPreview *)(p))
#define WIN_PREVIEW_DATA_KEY	"win_preview_data"
#define WIN_PREVIEW_GET_DATA(w)	(((w) != NULL) ?	\
 WIN_PREVIEW(gtk_object_get_data(			\
  GTK_OBJECT(w), WIN_PREVIEW_DATA_KEY			\
 )) : NULL)						\


static void WinPreviewDestroyCB(gpointer data);
static gint WinPreviewEventCB(
	GtkWidget *widget, GdkEvent *event, gpointer data
);
static void WinPreviewRealizeCB(GtkWidget *widget, gpointer data);
static gint WinPreviewDrawNextFrameTOCB(gpointer data);
#if defined(HAVE_IMAGE_MAGICK)
static MagickBooleanType WinPreviewImageMagickProgressCB(
	const char *text,
	const MagickOffsetType offset,
	const MagickSizeType span,
	void *data
);
#endif

static void WinPreviewDraw(WinPreview *pv);

GtkWidget *WinPreviewNew(void);
void WinPreviewSetSize(GtkWidget *w, gint width, gint height);
void WinPreviewSetShowDetails(GtkWidget *w, gboolean show);
void WinPreviewQueueDraw(GtkWidget *w);
GtkWidget *WinPreviewGetDrawingArea(GtkWidget *w);
static void WinPreviewSetImageFileIterate(
	WinPreview *pv, MagickWand *wand,
	gint max_width, gint max_height,
	gboolean overlay_prev,
	GdkWindow *window
);
#ifdef HAVE_LIBENDEAVOUR2
void WinPreviewSetImageFile(
	GtkWidget *w, const gchar *path,
	EDVContext *edv_ctx
);
#else
void WinPreviewSetImageFile(GtkWidget *w, const gchar *path);
#endif
void WinPreviewClear(GtkWidget *w);


#define ATOI(s)		(((s) != NULL) ? atoi(s) : 0)
#define ATOL(s)		(((s) != NULL) ? atol(s) : 0)
#define ATOF(s)		(((s) != NULL) ? atof(s) : 0.0f)
#define STRDUP(s)	(((s) != NULL) ? g_strdup(s) : NULL)

#define MAX(a,b)	(((a) > (b)) ? (a) : (b))
#define MIN(a,b)	(((a) < (b)) ? (a) : (b))
#define CLIP(a,l,h)	(MIN(MAX((a),(l)),(h)))
#define STRLEN(s)	(((s) != NULL) ? strlen(s) : 0)
#define STRISEMPTY(s)	(((s) != NULL) ? (*(s) == '\0') : TRUE)

#ifdef HAVE_IMAGE_MAGICK
#define MAGICK_FREE(p)		(((p) != NULL) ? \
 MagickRelinquishMemory(p) : NULL)
#endif


/*
 *	GtkObject data destroy callback.
 *
 *	Called when the Preview's toplevel GtkWidget is destroyed.
 */
static void WinPreviewDestroyCB(gpointer data)
{
	GList *glist;
	WinPreview *pv = WIN_PREVIEW(data);
	if(pv == NULL)
	    return;

	GTK_TIMEOUT_REMOVE(pv->draw_next_frame_toid);
	pv->draw_next_frame_toid = 0;

	GDK_PIXMAP_UNREF(pv->pixmap);

	for(glist = pv->img_pixmap_list;
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	    GDK_PIXMAP_UNREF((GdkPixmap *)glist->data);
	g_list_free(pv->img_pixmap_list);
	for(glist = pv->img_mask_list; 
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	    GDK_BITMAP_UNREF((GdkBitmap *)glist->data);
	g_list_free(pv->img_mask_list);

	g_list_free(pv->img_delay_list);

	g_free(pv->img_label);

	g_free(pv->img_file);

	g_free(pv);
}


/*
 *	GtkDrawingArea event signal callback.
 */
static gint WinPreviewEventCB(
	GtkWidget *widget, GdkEvent *event, gpointer data  
)
{
	gint status = FALSE;
	GdkEventConfigure *configure;
	GdkEventFocus *focus;
	WinPreview *pv = WIN_PREVIEW(data);
	if((widget == NULL) || (event == NULL) || (pv == NULL))
	    return(status);

	switch((gint)event->type)
	{
	  case GDK_CONFIGURE:
	    configure = (GdkEventConfigure *)event;
	    GDK_PIXMAP_UNREF(pv->pixmap);
	    pv->pixmap = gdk_pixmap_new(
		configure->window, configure->width, configure->height,
		-1
	    );
	    status = TRUE;
	    break;

	  case GDK_EXPOSE:
	    WinPreviewDraw(pv);
	    status = TRUE;
	    break;

	  case GDK_FOCUS_CHANGE:
	    focus = (GdkEventFocus *)event;
	    if(focus->in)
		GTK_WIDGET_SET_FLAGS(widget, GTK_HAS_FOCUS);
	    else
		GTK_WIDGET_UNSET_FLAGS(widget, GTK_HAS_FOCUS);
	    gtk_widget_queue_draw(widget);
	    status = TRUE;
	    break;

	}

	return(status);
}

/*
 *	GtkDrawingArea "realized" signal callback.
 */
static void WinPreviewRealizeCB(GtkWidget *widget, gpointer data)
{
	WinPreview *pv = WIN_PREVIEW(data);
	if(pv == NULL)
	    return;


}

/*
 *	Draw next frame timeout callback.
 */
static gint WinPreviewDrawNextFrameTOCB(gpointer data)
{
	gint nframes;
	guint delay;
	GList *glist;
	GtkWidget *w;
	WinPreview *pv = WIN_PREVIEW(data);
	if(pv == NULL)
	    return(FALSE);

	/* Go to next frame */
	pv->frame_num++;

	/* Frames cycled? */
	nframes = g_list_length(pv->img_pixmap_list);
	if(pv->frame_num >= nframes)
	{
/* TODO: Check number of times to repeat instead of repeat infinately */
	    pv->frame_num = 0;
	}

	/* Get this frame's delay */
	glist = g_list_nth(pv->img_delay_list, (guint)pv->frame_num);
	delay = (glist != NULL) ? (guint)glist->data : 0;

	w = pv->drawing_area;
	if(w != NULL)
	    gtk_widget_queue_draw(w);

	/* Schedual next frame draw */
	pv->draw_next_frame_toid = gtk_timeout_add(
	    delay,
	    WinPreviewDrawNextFrameTOCB, pv
	);

	return(FALSE);
}


#if defined(HAVE_IMAGE_MAGICK)
/*
 *	Image Magick progress callback.
 */
static MagickBooleanType WinPreviewImageMagickProgressCB(
	const char *text,
	const MagickOffsetType offset,
	const MagickSizeType span,    
	void *data
)
{
	WinPreview *pv = WIN_PREVIEW(data);
	if(pv == NULL)
	    return(MagickFalse);

	return(MagickTrue);
}
#endif


/*
 *	Redraws the image and label on to the Preview's back GdkPixmap
 *	buffer and puts the Gdkpixmap to the GtkDrawingArea's GdkWindow.
 */
static void WinPreviewDraw(WinPreview *pv)
{
	const gint border_major = 5;
	gint x, y, width, height;
	GdkFont *font;
	GdkWindow *window;
	GdkDrawable *drawable;
	GtkStateType state;
	GtkStyle *style;
	GtkWidget *w;

	if(pv == NULL)
	    return;

	w = pv->drawing_area;
	window = (w != NULL) ? w->window : NULL;
	if(window == NULL)
	    return;

	drawable = (GdkDrawable *)pv->pixmap;
	if(drawable == NULL)
	    drawable = (GdkDrawable *)window;

	state = GTK_WIDGET_STATE(w);
	style = gtk_widget_get_style(w);
	gdk_window_get_size(window, &width, &height);
	if((style == NULL) || (width <= 0) || (height <= 0))
	    return;

	font = style->font;

	/* Draw background */
	gdk_draw_rectangle(  
	    drawable, style->base_gc[state], TRUE,
	    0, 0, width, height
	);

	x = 0;
	y = 0;

	/* Draw image */
	if(pv->img_pixmap_list != NULL)
	{
	    const gint	frame_num = MAX(pv->frame_num, 0),
			nframes = g_list_length(pv->img_pixmap_list);
	    if(frame_num < nframes)
	    {
		GdkPixmap *pixmap;
		GdkBitmap *mask;
		GList *glist = g_list_nth(
		    pv->img_pixmap_list, (guint)frame_num
		);
		pixmap = (glist != NULL) ? (GdkPixmap *)glist->data : NULL;
		glist = g_list_nth(
		    pv->img_mask_list, (guint)frame_num  
		);
		mask = (glist != NULL) ? (GdkBitmap *)glist->data : NULL;
		if(pixmap != NULL)
		{
		    gint img_width, img_height;
		    GdkGC *gc = style->fg_gc[state];

		    gdk_window_get_size(pixmap, &img_width, &img_height);

		    x = (pv->img_label != NULL) ?
			border_major : ((width - img_width) / 2);
		    y = (height - img_height) / 2;

		    gdk_gc_set_clip_mask(gc, mask);
		    gdk_gc_set_clip_origin(gc, x, y);
		    gdk_draw_pixmap(
			drawable, gc, pixmap,
			0, 0, x, y, img_width, img_height
		    );
		    gdk_gc_set_clip_mask(gc, NULL);

		    x += img_width + border_major;
		}
	    }
	}

	/* Draw label */
	if((pv->img_label != NULL) && (font != NULL))
	{
	    const gint font_height = font->ascent + font->descent;
	    const gchar *s, *s_end;
	    gint len, label_height, nlines = 1;
	    GdkTextBounds b;
	    GdkGC *gc = style->text_gc[state];

	    for(s = pv->img_label; *s != '\0'; s++)
	    {
		if(*s == '\n')
		    nlines++;
	    }

	    label_height = nlines * font_height;
	    y = (height - label_height) / 2;

	    s = pv->img_label;
	    while(s != NULL)
	    {
		s_end = strchr(s, '\n');
		len = (s_end != NULL) ? (gint)(s_end - s) : STRLEN(s);
		if(len > 0)
		{
		    gdk_text_bounds(font, s, len, &b);
		    gdk_draw_text(
			drawable, font, gc,
			x - b.lbearing,
			y + font->ascent,
			s, len
		    );
		}
		y += font_height;
		s = (s_end != NULL) ? (s_end + 1) : NULL;
	    }
	}

	/* Send drawable to window if drawable is not the window */
	if(drawable != window)
	    gdk_draw_pixmap(
		window, style->fg_gc[state], drawable,
		0, 0, 0, 0, width, height
	    );
}


/*
 *	Creates a new Preview.
 */
GtkWidget *WinPreviewNew(void)
{
	GtkWidget *w, *parent;
	WinPreview *pv = WIN_PREVIEW(g_malloc0(
	    sizeof(WinPreview)
	));
	if(pv == NULL)
	    return(NULL);

	pv->pixmap = NULL;
	pv->img_pixmap_list = NULL;
	pv->img_mask_list = NULL;
	pv->img_delay_list = NULL;
	pv->frame_num = 0;
	pv->show_details = FALSE;
	pv->img_label = NULL;
	pv->img_file = NULL;
	pv->draw_next_frame_toid = 0;

	pv->toplevel = w = gtk_frame_new(NULL);
	gtk_object_set_data_full(
	    GTK_OBJECT(w), WIN_PREVIEW_DATA_KEY,
	    pv, WinPreviewDestroyCB
	);
	gtk_frame_set_shadow_type(GTK_FRAME(w), GTK_SHADOW_IN);
	parent = w;

	pv->drawing_area = w = gtk_drawing_area_new();
	GTK_WIDGET_SET_FLAGS(w, GTK_CAN_FOCUS);
	gtk_widget_add_events(
	    w,
	    GDK_STRUCTURE_MASK | GDK_EXPOSURE_MASK |
	    GDK_FOCUS_CHANGE_MASK |
	    GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK |
	    GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "expose_event",
	    GTK_SIGNAL_FUNC(WinPreviewEventCB), pv
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "configure_event",
	    GTK_SIGNAL_FUNC(WinPreviewEventCB), pv
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "focus_in_event",
	    GTK_SIGNAL_FUNC(WinPreviewEventCB), pv
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "focus_out_event",
	    GTK_SIGNAL_FUNC(WinPreviewEventCB), pv
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_press_event",
	    GTK_SIGNAL_FUNC(WinPreviewEventCB), pv
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "button_release_event",
	    GTK_SIGNAL_FUNC(WinPreviewEventCB), pv
	);
	gtk_signal_connect(
	    GTK_OBJECT(w), "realize",
	    GTK_SIGNAL_FUNC(WinPreviewRealizeCB), pv
	);
	gtk_container_add(GTK_CONTAINER(parent), w);
	gtk_widget_show(w);

	return(pv->toplevel);
}

/*
 *	Sets the Preview's GtkDrawingArea size.
 */
void WinPreviewSetSize(GtkWidget *w, gint width, gint height)
{
	WinPreview *pv = WIN_PREVIEW_GET_DATA(w);
	if(pv == NULL)
	    return;

	w = pv->drawing_area;
	if(w != NULL)
	    gtk_widget_set_usize(w, width, height);
}

/*
 *	Sets the Preview to show details or not.
 */
void WinPreviewSetShowDetails(GtkWidget *w, gboolean show)
{ 
	WinPreview *pv = WIN_PREVIEW_GET_DATA(w);
	if(pv == NULL)
	    return;

	if(pv->show_details == show)
	    return;

	pv->show_details = show;

	if(pv->img_file != NULL)
	{
	    gchar *path = STRDUP(pv->img_file);
#ifdef HAVE_LIBENDEAVOUR2
	    WinPreviewSetImageFile(w, path, pv->edv_ctx);
#else
	    WinPreviewSetImageFile(w, path);
#endif
	    g_free(path);
	}
}

/*
 *	Queues the Preview's GtkDrawingArea to draw.
 */
void WinPreviewQueueDraw(GtkWidget *w)
{
	WinPreview *pv = WIN_PREVIEW_GET_DATA(w);
	if(pv == NULL)
	    return;

	w = pv->drawing_area;
	if(w != NULL)
	    gtk_widget_queue_draw(w);
}

/*
 *	Returns the Preview's GtkDrawingArea.
 */
GtkWidget *WinPreviewGetDrawingArea(GtkWidget *w)
{
	WinPreview *pv = WIN_PREVIEW_GET_DATA(w);
	if(pv == NULL)
	    return(NULL);

	return(pv->drawing_area);
}

/*
 *	Called by WinPreviewSetImageFile() to append the image contained
 *	in MagickWand to the WinPreview.
 *
 *	The max_width and max_height specify the maximum size of the
 *	image, if the image contained in the MagickWand is larger than
 *	this size then it will be resized to fit.
 *
 *	If overlay_prev is TRUE then the previous WinPreview image's
 *	data (if any) will be drawn to the new image and then the new
 *	image's data will be overlayed.
 */
static void WinPreviewSetImageFileIterate(
	WinPreview *pv, MagickWand *wand,
	gint max_width, gint max_height,
	gboolean overlay_prev,
	GdkWindow *window
)
{
	gint	img_width = (gint)MagickGetImageWidth(wand),
		img_height = (gint)MagickGetImageHeight(wand),
		new_width,
		new_height;
	const gfloat aspect = (img_height > 0) ?
	    ((gfloat)img_width / (gfloat)img_height) : 1.0f;

	/* Check if we need to shrink the image to fit in the specified
	 * max size
	 *
	 * Calculate the new image size
	 */
	if((img_width > max_width) || (img_height > max_height))
	{
	    new_width = max_width;
	    new_height = (aspect > 0.0f) ?
		(new_width / aspect) : new_width;
	    if(new_height > max_height)
	    {
		new_height = max_height;
		new_width = new_height * aspect;
	    }
	}
	else
	{
	    new_width = img_width;
	    new_height = img_height;
	}
	if(new_width < 1)
	    new_width = 1;
	if(new_height < 1)
	    new_height = 1;

	/* Resize image? */
	if((img_width != new_width) || (img_height != new_height))
	{
	    if(MagickResizeImage(
		wand,
		(unsigned long)new_width,	/* New width */
		(unsigned long)new_height,	/* New height */
		PointFilter,			/* Filter type */
/* We do not need and cannot use filters that play with the alpha
 * channel
		LanczosFilter,
 */
		1.0
	    ) != MagickFalse)
	    {
		img_width = new_width = (gint)MagickGetImageWidth(wand);
		img_height = new_height = (gint)MagickGetImageHeight(wand);
	    }
	}

	/* Get the image data based on the image type, then create a
	 * GdkPixmap and (optional) GdkBitmap and put the image data to
	 * them
	 */
	switch(MagickGetImageType(wand))
	{
	    gint bpp, bpl;
	    guint8 *data;
	  case GrayscaleMatteType:
	  case PaletteMatteType:
	  case TrueColorMatteType:
	  case ColorSeparationMatteType:
	    bpp = 4;
	    bpl = new_width * bpp;
	    data = (guint8 *)g_malloc(bpl * new_height);
	    if(data != NULL)
	    {
		if(MagickGetImagePixels(
		    wand,
		    0, 0,
		    (unsigned long)new_width,
		    (unsigned long)new_height,
		    "RGBA",
		    CharPixel,
		    (unsigned char *)data
		) != MagickFalse)
	        {
		    GdkPixmap *pixmap = gdk_pixmap_new(
		        window, new_width, new_height, -1
		    );
		    GdkBitmap *mask = GUICreateBitmapFromDataRGBA(
			new_width, new_height,
			bpl, data, 0x80, window
		    );
		    GdkGC *gc = gdk_gc_new(window);
		    if(overlay_prev)
		    {
			GList *glist = g_list_last(pv->img_pixmap_list);
			GdkPixmap *prev_pixmap = (glist != NULL) ?
			    (GdkPixmap *)glist->data : NULL;
			if(prev_pixmap != NULL)
			{
			    gint prev_width, prev_height;
			    gdk_window_get_size(
				prev_pixmap, &prev_width, &prev_height
			    );
			    gdk_draw_pixmap(
				(GdkDrawable *)pixmap, gc, prev_pixmap,
				0, 0, 0, 0, prev_width, prev_height
			    );
			    gdk_gc_set_clip_origin(gc, 0, 0);
			    gdk_gc_set_clip_mask(gc, mask);
			}
		    }
		    gdk_draw_rgb_32_image(
		        (GdkDrawable *)pixmap, gc,
		        0, 0, new_width, new_height,
		        GDK_RGB_DITHER_NORMAL,
		        (guchar *)data,
		        bpl
		    );
		    gdk_gc_set_clip_mask(gc, NULL);
		    pv->img_pixmap_list = g_list_append(
			pv->img_pixmap_list, pixmap
		    );

		    /* Get rid of the mask if we overlayed with the
		     * previous image since there would be no
		     * transparency
		     */
		    if(overlay_prev)
		    {
			GDK_BITMAP_UNREF(mask);
			mask = NULL;
		    }
		    pv->img_mask_list = g_list_append(
			pv->img_mask_list, mask
		    );

		    GDK_GC_UNREF(gc);
	        }
	        g_free(data);
	    }
	    break;
	  default:
	    bpp = 3;
	    bpl = new_width * bpp;
	    data = (guint8 *)g_malloc(bpl * new_height);
	    if(data != NULL)
	    {
		if(MagickGetImagePixels(
		    wand,
		    0, 0,
		    (unsigned long)new_width,
		    (unsigned long)new_height,
		    "RGB",
		    CharPixel,
		    (unsigned char *)data
		) != MagickFalse)
		{
		    GdkPixmap *pixmap = gdk_pixmap_new(
			window, new_width, new_height, -1
		    );
		    GdkGC *gc = gdk_gc_new(window);
		    gdk_draw_rgb_image(
			(GdkDrawable *)pixmap, gc,
			0, 0, new_width, new_height,
			GDK_RGB_DITHER_NORMAL,
			(guchar *)data,
			bpl
		    );
		    pv->img_pixmap_list = g_list_append(  
			pv->img_pixmap_list, pixmap
		    );
		    pv->img_mask_list = g_list_append(
			pv->img_mask_list, NULL
		    );
		    GDK_GC_UNREF(gc);
		}
		g_free(data);
	    }
	    break;
	}

	/* Set image delay */
	pv->img_delay_list = g_list_append(
	    pv->img_delay_list,
	    (gpointer)((guint)MagickGetImageDelay(wand) * 10)
	);

}

/*
 *	Sets a new image specified by path to the Preview and queues a
 *	draw.
 */
#ifdef HAVE_LIBENDEAVOUR2
void WinPreviewSetImageFile(
	GtkWidget *w, const gchar *path,
	EDVContext *edv_ctx
)
#else
void WinPreviewSetImageFile(
	GtkWidget *w, const gchar *path
)
#endif
{
	struct stat stat_buf;
#ifdef HAVE_IMAGE_MAGICK
	MagickWand *wand;
#endif
	gint width, height;
	GList *glist;
	GdkWindow *window;
	WinPreview *pv = WIN_PREVIEW_GET_DATA(w); 
	if((pv == NULL) || STRISEMPTY(path))
	    return;

	w = pv->drawing_area;
	window = (w != NULL) ? w->window : NULL;
	if(window == NULL)
	    return;

	/* Get width and height as the maximum size */
	gdk_window_get_size(window, &width, &height);


	/* Remove the draw next frame timeout */
	GTK_TIMEOUT_REMOVE(pv->draw_next_frame_toid);
	pv->draw_next_frame_toid = 0;

	/* Unload the existing image */
	for(glist = pv->img_pixmap_list; 
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	    GDK_PIXMAP_UNREF((GdkPixmap *)glist->data);
	g_list_free(pv->img_pixmap_list);
	pv->img_pixmap_list = NULL;

	for(glist = pv->img_mask_list; 
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	    GDK_BITMAP_UNREF((GdkBitmap *)glist->data);
	g_list_free(pv->img_mask_list);
	pv->img_mask_list = NULL;

	g_list_free(pv->img_delay_list);
	pv->img_delay_list = NULL;

	pv->frame_num = 0;

	g_free(pv->img_label);
	pv->img_label = NULL;

	g_free(pv->img_file);
	pv->img_file = STRDUP(path);

#ifdef HAVE_LIBENDEAVOUR2
	pv->edv_ctx = edv_ctx;
#endif  


	/* Get the image file's stats */
	if(stat(path, &stat_buf))
	    memset(&stat_buf, 0x00, sizeof(struct stat));

#ifdef HAVE_IMAGE_MAGICK
	wand = NewMagickWand();
	if(wand != NULL)
	{
	    MagickSetProgressMonitor(
		wand, WinPreviewImageMagickProgressCB, pv
	    );

	    /* Read image */
	    if(MagickReadImage(wand, path) != MagickFalse)
	    {
		const gint	border_major = 5,
				orig_width = (gint)MagickGetImageWidth(wand),
				orig_height = (gint)MagickGetImageHeight(wand),
				nframes = (gint)MagickGetNumberImages(wand);
		gint i;
		DisposeType	disposal_method = UndefinedDispose,
				prev_disposal_method;
		GtkStyle *style = gtk_widget_get_style(w);
		GdkFont *font = (style != NULL) ? style->font : NULL;

#if 0
g_print(
 "compression_type=%i compression_quality=%i\n",
 (gint)MagickGetImageCompression(wand),
 (gint)MagickGetImageCompressionQuality(wand)
);
#endif

		/* Format label? */
		if(pv->show_details)
		{
		    const gulong size = (gulong)stat_buf.st_size;
		    gchar *nframes_str = (nframes > 1) ?
			nframes_str = g_strdup_printf(
			    "%i frames\n", nframes
			) : NULL;
#ifdef HAVE_LIBENDEAVOUR2
		    pv->img_label = g_strdup_printf(
"%s\n\
%ix%i\n%s\
%s %s\n\
%s",
		        g_basename(path),
		        orig_width, orig_height,
		        (nframes_str != NULL) ? nframes_str : "",
		        edv_str_size_delim(stat_buf.st_size),
		        (size == 1l) ? "byte" : "bytes",
		        edv_date_format(edv_ctx, (gulong)stat_buf.st_mtime)
		    );
#else
		    pv->img_label = g_strdup_printf(
"%s\n\
%ix%i\n%s\
%ld %s",
			g_basename(path),
			orig_width, orig_height,
			(nframes_str != NULL) ? nframes_str : "",
			size,
			(size == 1l) ? "byte" : "bytes"
		    );
#endif
		    g_free(nframes_str);
		}


		/* Reduce the image's maximum size */

		/* Account for border */
		width -= (2 * border_major);
		height -= (2 * border_major);

		/* Account for label */
		if((pv->img_label != NULL) && (font != NULL))
		{
		    GdkTextBounds b;
		    gint len, longest_line_width = 0;
		    const gchar *s = pv->img_label, *s_end;
		    while(s != NULL)
		    {
			s_end = strchr(s, '\n');
			len = (s_end != NULL) ? (gint)(s_end - s) : STRLEN(s);
			if(len > 0)
			{
			    gdk_text_bounds(font, s, len, &b);
			    if(b.width > longest_line_width)
				longest_line_width = b.width;
			}
			s = (s_end != NULL) ? (s_end + 1) : NULL;
		    }
		    width -= longest_line_width + border_major;
		}

		/* Make sure maximum image size is big enough */
		if(width < 20)
		    width = 20;
		if(height < 20)
		    height = 20;


		/* Load each frame */
		for(i = 0; i < nframes; i++)
		{
		    prev_disposal_method = disposal_method;
		    disposal_method = MagickGetImageDispose(wand);
		    WinPreviewSetImageFileIterate(
			pv, wand,
			width, height,		/* Max size */
			((i > 0) && (prev_disposal_method != BackgroundDispose)) ?
			    TRUE : FALSE,
			window
		    );
		    MagickNextImage(wand);
		}

		/* Need to schedual initial frame draw */
		if(nframes > 1)
		{
		    GList *glist = g_list_nth(pv->img_delay_list, 0);
		    guint delay = (glist != NULL) ?
			(guint)glist->data : 0;
		    pv->draw_next_frame_toid = gtk_timeout_add(
			delay,
			WinPreviewDrawNextFrameTOCB, pv
		    );
		}
	    }

	    DestroyMagickWand(wand);
	}
#endif

	w = pv->drawing_area;
	if(w != NULL)
	    gtk_widget_queue_draw(w);
}

/*
 *	Clears the Preview's image and queues a draw.
 */
void WinPreviewClear(GtkWidget *w)
{
	GList *glist;
	WinPreview *pv = WIN_PREVIEW_GET_DATA(w);
	if(pv == NULL)
	    return;

	GTK_TIMEOUT_REMOVE(pv->draw_next_frame_toid);
	pv->draw_next_frame_toid = 0;

	for(glist = pv->img_pixmap_list;
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	    GDK_PIXMAP_UNREF((GdkPixmap *)glist->data);
	g_list_free(pv->img_pixmap_list);
	pv->img_pixmap_list = NULL;

	for(glist = pv->img_mask_list;
	    glist != NULL;
	    glist = g_list_next(glist)
	)
	    GDK_BITMAP_UNREF((GdkBitmap *)glist->data);
	g_list_free(pv->img_mask_list);
	pv->img_mask_list = NULL;

	g_list_free(pv->img_delay_list);
	pv->img_delay_list = NULL;

	pv->frame_num = 0;

	g_free(pv->img_label);
	pv->img_label = NULL;

	g_free(pv->img_file);
	pv->img_file = NULL;

#ifdef HAVE_LIBENDEAVOUR2
	pv->edv_ctx = NULL;
#endif

	w = pv->drawing_area;
	if(w != NULL)
	    gtk_widget_queue_draw(w);
}
